@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: configuration.ml,v 1.35 2012-05-14 14:03:36 tews Exp $
+ * $Id: configuration.ml,v 1.38 2013/01/18 16:39:10 tews Exp $
  *)
 
 
@@ -48,7 +48,7 @@ let config_file_location =
     configuration file is (mostly) a marshaled configuration record.
     In order to be independent of Gdk marshaling, the configuration
     record consists only of pure OCaml values. Fonts and colors are
-    therefore not accessed via the configuration record, but via there
+    therefore not accessed via the configuration record, but via their
     own references (of some suitable Gdk type). These references must,
     of course, be kept in sync with the current configuration. All
     other configurable values are accessed through the current
@@ -100,6 +100,12 @@ type t = {
   level_distance : int;
   (** Vertical distance between two levels of the proof tree. *)
 
+  proof_tree_sep : int;
+  (** Horizontal distance between two independent proof trees in one layer *)
+
+  layer_sep : int;
+  (** Vertical distance between two layers of proof trees *)
+
   button_1_drag_acceleration : float;
   (** Acceleration multiplier for dragging the proof-tree display
       inside its viewport. Positive values move the viewport (i.e.,
@@ -191,6 +197,9 @@ type t = {
   node_window_max_lines : int;
   (** Maximal number of text lines in external node windows. *)
 
+  ext_table_lines : int;
+  (** Default number of lines for the table of existential variables. *)
+
   debug_mode : bool;
   (** Print more exception backtraces for internal errors, if true. *)
 
@@ -231,6 +240,8 @@ let default_configuration =
     subtree_sep = 5;
     line_sep = 3;
     level_distance = 38;
+    proof_tree_sep = 15;
+    layer_sep = 30;
 
     turnstile_left_bar_x_offset = 0;
     turnstile_left_bar_y_offset = 0;
@@ -262,6 +273,7 @@ let default_configuration =
 
     internal_sequent_window_lines = 1;
     node_window_max_lines = 35;
+    ext_table_lines = 8;
 
     debug_mode = false;
     copy_input = false;
@@ -435,7 +447,7 @@ let geometry_string = ref ""
 let config_file_header_start = "Prooftree configuration file version "
 
 (** Version specific header of the current config file version. *)
-let config_file_version = "03"
+let config_file_version = "04"
 
 (** The complete ASCII header of configuration files. *)
 let config_file_header = config_file_header_start ^ config_file_version ^ "\n"
@@ -510,12 +522,16 @@ let config_window = ref None
                                 for turnstile size
     - line_sep_spinner 		{xref lablgtk class GEdit.spin_button} 
                                 for line gap
+    - proof_tree_sep_spinner    {xref lablgtk class GEdit.spin_button}
+                                for proof tree sep
     - subtree_sep_spinner 	{xref lablgtk class GEdit.spin_button} 
                                 for node padding
     - command_length_spinner 	{xref lablgtk class GEdit.spin_button} 
                                 for command length
     - level_dist_spinner 	{xref lablgtk class GEdit.spin_button} 
                                 for vertical distance
+    - layer_sep_spinner 	{xref lablgtk class GEdit.spin_button} 
+                                layer sep
     - tree_font_button		{xref lablgtk class GButton.font_button} 
                                 for proof tree font
     - sequent_font_button	{xref lablgtk class GButton.font_button} 
@@ -550,6 +566,8 @@ let config_window = ref None
                                   for lines in the internal sequent window
     - external_node_lines_spinner {xref lablgtk class GEdit.spin_button}
                                   for lines in external node windows
+    - ext_table_lines_spinner     {xref lablgtk class GEdit.spin_button}
+                                  for lines in evar table
     - debug_check_box		{xref lablgtk class GButton.toggle_button}
                                 for the more-debug-info check box
     - tee_file_box_check_box 	{xref lablgtk class GButton.toggle_button}
@@ -570,9 +588,11 @@ class config_window
   line_width_spinner
   turnstile_size_spinner
   line_sep_spinner
+  proof_tree_sep_spinner
   subtree_sep_spinner
   command_length_spinner
   level_dist_spinner
+  layer_sep_spinner
   tree_font_button
   sequent_font_button
   current_color_button
@@ -590,6 +610,7 @@ class config_window
   default_size_width_spinner default_size_height_spinner
   internal_seq_lines_spinner
   external_node_lines_spinner
+  ext_table_lines_spinner
   debug_check_box
   tee_file_box_check_box 
   tee_file_name_label tee_file_name_entry tee_file_name_button
@@ -608,6 +629,11 @@ object (self)
   (** {xref lablgtk class GData.adjustment} of the line-gap spin button. *)
   val line_sep_adjustment = line_sep_spinner#adjustment
 
+  (** {xref lablgtk class GData.adjustment} of the proof_tree_sep spin
+      button. 
+  *)
+  val proof_tree_sep_adjustment = proof_tree_sep_spinner#adjustment
+
   (** {xref lablgtk class GData.adjustment} of the node-padding spin
       button.
   *)
@@ -623,6 +649,11 @@ object (self)
   *)
   val level_dist_adjustment = level_dist_spinner#adjustment
 
+  (** {xref lablgtk class GData.adjustment} of the layer_sep spin
+      button. 
+  *)
+  val layer_sep_adjustment = layer_sep_spinner#adjustment
+
   (** {xref lablgtk class GData.adjustment} of the drag-acceleration
       spin button.
   *)
@@ -648,6 +679,11 @@ object (self)
   *)
   val external_node_lines_adjustment = external_node_lines_spinner#adjustment
 
+  (** {xref lablgtk class GData.adjustment} of the spin button for the
+      default number of lines in the existential variable table. 
+  *)
+  val ext_table_lines_adjustment = ext_table_lines_spinner#adjustment
+
   (** Make this configuration dialog visible. *)
   method present = top_window#present()
 
@@ -659,8 +695,10 @@ object (self)
     turnstile_size_adjustment#set_value (float_of_int conf.turnstile_radius);
     subtree_sep_adjustment#set_value (float_of_int conf.subtree_sep);
     line_sep_adjustment#set_value (float_of_int conf.line_sep);
+    proof_tree_sep_adjustment#set_value (float_of_int conf.proof_tree_sep);
     command_length_adjustment#set_value (float_of_int conf.proof_command_length);
     level_dist_adjustment#set_value (float_of_int conf.level_distance);
+    layer_sep_adjustment#set_value (float_of_int conf.layer_sep);
     tree_font_button#set_font_name conf.proof_tree_font;
     sequent_font_button#set_font_name conf.sequent_font;
     current_color_button#set_color (GDraw.color (`RGB conf.current_color));
@@ -691,6 +729,8 @@ object (self)
       (float_of_int conf.internal_sequent_window_lines);
     external_node_lines_adjustment#set_value
       (float_of_int conf.node_window_max_lines);
+    ext_table_lines_adjustment#set_value
+      (float_of_int conf.ext_table_lines);
     debug_check_box#set_active conf.debug_mode;
     tee_file_box_check_box#set_active conf.copy_input;
     tee_file_name_entry#set_text conf.copy_input_file;
@@ -755,12 +795,14 @@ object (self)
   method private extract_configuration =
     let round_to_int f = int_of_float(f +. 0.5) in
     let c = {
-      turnstile_radius = round_to_int turnstile_size_adjustment#value;
       turnstile_line_width = round_to_int line_width_adjustment#value;
-      proof_command_length = round_to_int command_length_adjustment#value;
-      subtree_sep = round_to_int subtree_sep_adjustment#value;
+      turnstile_radius = round_to_int turnstile_size_adjustment#value;
       line_sep = round_to_int line_sep_adjustment#value;
+      proof_tree_sep = round_to_int proof_tree_sep_adjustment#value;
+      subtree_sep = round_to_int subtree_sep_adjustment#value;
+      proof_command_length = round_to_int command_length_adjustment#value;
       level_distance = round_to_int level_dist_adjustment#value;
+      layer_sep = round_to_int layer_sep_adjustment#value;
 
       turnstile_left_bar_x_offset = 0;
       turnstile_left_bar_y_offset = 0;
@@ -810,6 +852,8 @@ object (self)
 	round_to_int internal_seq_lines_adjustment#value;
       node_window_max_lines = 
 	round_to_int external_node_lines_adjustment#value;
+      ext_table_lines =
+	round_to_int ext_table_lines_adjustment#value;
 
       debug_mode = debug_check_box#active;
       copy_input = tee_file_box_check_box#active;
@@ -1035,6 +1079,21 @@ let make_config_window () =
   line_sep_label#misc#set_tooltip_text line_sep_tooltip;
   line_sep_spinner#misc#set_tooltip_text line_sep_tooltip;
 
+  (* proof_tree_sep *)
+  let proof_tree_sep_tooltip = 
+    "Additional padding between adjacent proof trees in one layer" in
+  let proof_tree_sep_label = GMisc.label
+    ~text:"Tree padding" ~xalign:0.0 ~xpad:5
+    ~packing:(tree_frame_table#attach ~left:0 ~top:3) () in
+  let proof_tree_sep_spinner = GEdit.spin_button
+    ~digits:0 ~numeric:true
+    ~packing:(tree_frame_table#attach ~left:1 ~top:3) () in
+  adjustment_set_pos_int ~lower:0.0 proof_tree_sep_spinner#adjustment;
+  proof_tree_sep_spinner#adjustment#set_value
+    (float_of_int !current_config.proof_tree_sep);
+  proof_tree_sep_label#misc#set_tooltip_text proof_tree_sep_tooltip;
+  proof_tree_sep_spinner#misc#set_tooltip_text proof_tree_sep_tooltip;
+
   (* subtree_sep *)
   let subtree_sep_tooltip =
     "Additional padding added to the width of each node in the proof tree" in
@@ -1068,7 +1127,7 @@ let make_config_window () =
   (* level distance *)
   let level_dist_tooltip = "Vertical distance between neighboring nodes" in
   let level_dist_label = GMisc.label
-    ~text:"Vertical distance" ~xpad:5
+    ~text:"Vertical distance" ~xalign:0.0 ~xpad:5
     ~packing:(tree_frame_table#attach ~left:3 ~top:2) () in
   let level_dist_spinner = GEdit.spin_button
     ~digits:0 ~numeric:true
@@ -1079,6 +1138,21 @@ let make_config_window () =
   level_dist_label#misc#set_tooltip_text level_dist_tooltip;
   level_dist_spinner#misc#set_tooltip_text level_dist_tooltip;
 
+  (* layer_sep *)
+  let layer_sep_tooltip = 
+    "Additional padding between adjacent layers of proof trees" in
+  let layer_sep_label = GMisc.label
+    ~text:"Layer padding" ~xalign:0.0 ~xpad:5
+    ~packing:(tree_frame_table#attach ~left:3 ~top:3) () in
+  let layer_sep_spinner = GEdit.spin_button
+    ~digits:0 ~numeric:true
+    ~packing:(tree_frame_table#attach ~left:4 ~top:3) () in
+  adjustment_set_pos_int layer_sep_spinner#adjustment;
+  layer_sep_spinner#adjustment#set_value
+    (float_of_int !current_config.layer_sep);
+  layer_sep_label#misc#set_tooltip_text layer_sep_tooltip;
+  layer_sep_spinner#misc#set_tooltip_text layer_sep_tooltip;
+
   (****************************************************************************
    *
    * Fonts
@@ -1243,10 +1317,11 @@ let make_config_window () =
     ~packing:misc_frame#add () in
 
   (* doc tooltips *)
+  let misc_line = 0 in
   let doc_tooltip_tooltip = "Switch ordinary help tool tips on and off" in
   let doc_tooltip_alignment = GBin.alignment
     ~padding:(0,0,3,0)
-    ~packing:(misc_frame_table#attach ~left:0 ~right:2 ~top:0) () in
+    ~packing:(misc_frame_table#attach ~left:0 ~right:2 ~top:misc_line) () in
   let doc_tooltip_check_box = GButton.check_button
     ~label:"Display help tool tips"
     ~active:!current_config.display_doc_tooltips
@@ -1254,11 +1329,12 @@ let make_config_window () =
   doc_tooltip_alignment#misc#set_tooltip_text doc_tooltip_tooltip;
 
   (* turnstile tooltips *)
+  let misc_line = 1 in
   let turnstile_tooltip_tooltip = 
     "Switch sequent display as tool tip over the proof tree on and off" in
   let turnstile_tooltip_alignment = GBin.alignment
     ~padding:(0,0,3,0)
-    ~packing:(misc_frame_table#attach ~left:0 ~right:2 ~top:1) () in
+    ~packing:(misc_frame_table#attach ~left:0 ~right:2 ~top:misc_line) () in
   let turnstile_tooltip_check_box = GButton.check_button
     ~label:"Display turnstile tool tips"
     ~active:!current_config.display_turnstile_tooltips
@@ -1266,11 +1342,12 @@ let make_config_window () =
   turnstile_tooltip_alignment#misc#set_tooltip_text turnstile_tooltip_tooltip;
 
   (* command tooltips *)
+  let misc_line = 2 in
   let command_tooltip_tooltip = 
     "Switch display of truncated commands as tool tip on and off" in
   let command_tooltip_alignment = GBin.alignment
     ~padding:(0,0,3,0)
-    ~packing:(misc_frame_table#attach ~left:0 ~right:2 ~top:2) () in
+    ~packing:(misc_frame_table#attach ~left:0 ~right:2 ~top:misc_line) () in
   let command_tooltip_check_box = GButton.check_button
     ~label:"Display command tool tips"
     ~active:!current_config.display_command_tooltips
@@ -1278,14 +1355,15 @@ let make_config_window () =
   command_tooltip_alignment#misc#set_tooltip_text command_tooltip_tooltip;
 
   (* drag accel *)
+  let misc_line = 3 in
   let drag_accel_tooltip = 
     "Acceleration for dragging the viewport to the proof tree" in
   let drag_accel_label = GMisc.label
     ~text:"Drag acceleration" ~xalign:0.0 ~xpad:5
-    ~packing:(misc_frame_table#attach ~left:0 ~top:3) () in
+    ~packing:(misc_frame_table#attach ~left:0 ~top:misc_line) () in
   let drag_accel_spinner = GEdit.spin_button
     ~digits:2 ~numeric:true
-    ~packing:(misc_frame_table#attach ~left:1 ~top:3) () in
+    ~packing:(misc_frame_table#attach ~left:1 ~top:misc_line) () in
   drag_accel_spinner#adjustment#set_bounds
     ~lower:(-99.0) ~upper:99.0
     ~step_incr:0.01 ~page_incr:1.0 ();
@@ -1295,13 +1373,14 @@ let make_config_window () =
   drag_accel_spinner#misc#set_tooltip_text drag_accel_tooltip;
 
   (* default size *)
+  let misc_line = 4 in
   let default_size_tooltip = "Size for newly created proof tree windows" in
   let default_size_label = GMisc.label
     ~text:"Default window size" ~xalign:0.0 ~xpad:5
-    ~packing:(misc_frame_table#attach ~left:0 ~top:4) () in
+    ~packing:(misc_frame_table#attach ~left:0 ~top:misc_line) () in
   let default_size_width_spinner = GEdit.spin_button
     ~digits:0 ~numeric:true
-    ~packing:(misc_frame_table#attach ~left:1 ~top:4) () in
+    ~packing:(misc_frame_table#attach ~left:1 ~top:misc_line) () in
   default_size_width_spinner#adjustment#set_bounds
     ~lower:(-9999.0) ~upper:9999.0
     ~step_incr:1.0 ~page_incr:100.0 ();
@@ -1310,10 +1389,10 @@ let make_config_window () =
   let _x_label = GMisc.label
     ~text:"\195\151" (* multiplication sign U+00D7 *)
     ~xpad:5
-    ~packing:(misc_frame_table#attach ~left:2 ~top:4) () in
+    ~packing:(misc_frame_table#attach ~left:2 ~top:misc_line) () in
   let default_size_height_spinner = GEdit.spin_button
     ~digits:0 ~numeric:true
-    ~packing:(misc_frame_table#attach ~left:3 ~top:4) () in
+    ~packing:(misc_frame_table#attach ~left:3 ~top:misc_line) () in
   default_size_height_spinner#adjustment#set_bounds
     ~lower:(-9999.0) ~upper:9999.0
     ~step_incr:1.0 ~page_incr:100.0 ();
@@ -1324,16 +1403,17 @@ let make_config_window () =
   default_size_height_spinner#misc#set_tooltip_text default_size_tooltip;
 
   (* internal sequent window lines *)
+  let misc_line = 5 in
   let internal_seq_lines_tooltip = 
     "Initial height (in lines) of the sequent window \
      below the proof tree display" 
   in
   let internal_seq_lines_label = GMisc.label
     ~text:"Int. Sequent window" ~xalign:0.0 ~xpad:5
-    ~packing:(misc_frame_table#attach ~left:0 ~top:5) () in
+    ~packing:(misc_frame_table#attach ~left:0 ~top:misc_line) () in
   let internal_seq_lines_spinner = GEdit.spin_button
     ~digits:0 ~numeric:true
-    ~packing:(misc_frame_table#attach ~left:1 ~top:5) () in
+    ~packing:(misc_frame_table#attach ~left:1 ~top:misc_line) () in
   adjustment_set_pos_int ~lower:0.0 internal_seq_lines_spinner#adjustment;
   internal_seq_lines_spinner#adjustment#set_value
     (float_of_int !current_config.internal_sequent_window_lines);
@@ -1341,30 +1421,48 @@ let make_config_window () =
   internal_seq_lines_spinner#misc#set_tooltip_text internal_seq_lines_tooltip;
 
   (* external node window lines *)
+  let misc_line = 6 in
   let external_node_lines_tooltip = 
     "Maximal height (in lines) of additional node windows" in
   let external_node_lines_label = GMisc.label
     ~text:"Ext. node window" ~xalign:0.0 ~xpad:5
-    ~packing:(misc_frame_table#attach ~left:0 ~top:6) () in
+    ~packing:(misc_frame_table#attach ~left:0 ~top:misc_line) () in
   let external_node_lines_spinner = GEdit.spin_button
     ~digits:0 ~numeric:true
-    ~packing:(misc_frame_table#attach ~left:1 ~top:6) () in
+    ~packing:(misc_frame_table#attach ~left:1 ~top:misc_line) () in
   adjustment_set_pos_int external_node_lines_spinner#adjustment;
   external_node_lines_spinner#adjustment#set_value
     (float_of_int !current_config.node_window_max_lines);
   external_node_lines_label#misc#set_tooltip_text external_node_lines_tooltip;
   external_node_lines_spinner#misc#set_tooltip_text external_node_lines_tooltip;
 
+  (* existential table lines *)
+  let misc_line = 7 in
+  let ext_table_lines_tooltip = 
+    "Default number of lines for the existential variable table" in
+  let ext_table_lines_label = GMisc.label
+    ~text:"Evar table" ~xalign:0.0 ~xpad:5
+    ~packing:(misc_frame_table#attach ~left:0 ~top:misc_line) () in
+  let ext_table_lines_spinner = GEdit.spin_button
+    ~digits:0 ~numeric:true
+    ~packing:(misc_frame_table#attach ~left:1 ~top:misc_line) () in
+  adjustment_set_pos_int ext_table_lines_spinner#adjustment;
+  ext_table_lines_spinner#adjustment#set_value
+    (float_of_int !current_config.ext_table_lines);
+  ext_table_lines_label#misc#set_tooltip_text ext_table_lines_tooltip;
+  ext_table_lines_spinner#misc#set_tooltip_text ext_table_lines_tooltip;
+
   (* non-configurable config-file *)
+  let misc_line = 8 in
   let config_file_tooltip = 
     "The configuration file is determined at compilation time" in
   let config_file_label = GMisc.label
     ~text:"Configuration file"
     ~xalign:0.0 ~xpad:5
-    ~packing:(misc_frame_table#attach ~left:0 ~top:7) () in
+    ~packing:(misc_frame_table#attach ~left:0 ~top:misc_line) () in
   let config_file_alignment = GBin.alignment
     ~padding:(0,0,3,0)
-    ~packing:(misc_frame_table#attach ~left:1 ~right:4 ~top:7) () in
+    ~packing:(misc_frame_table#attach ~left:1 ~right:4 ~top:misc_line) () in
   let _config_file_file = GMisc.label
     ~text:config_file_location
     ~xalign:0.0
@@ -1450,9 +1548,11 @@ let make_config_window () =
       line_width_spinner
       turnstile_size_spinner
       line_sep_spinner
+      proof_tree_sep_spinner
       subtree_sep_spinner
       command_length_spinner
       level_dist_spinner
+      layer_sep_spinner
       tree_font_button
       sequent_font_button
       current_color_button
@@ -1470,15 +1570,18 @@ let make_config_window () =
       default_size_width_spinner default_size_height_spinner
       internal_seq_lines_spinner
       external_node_lines_spinner
+      ext_table_lines_spinner
       debug_check_box
       tee_file_box_check_box 
       tee_file_name_label tee_file_name_entry tee_file_name_button
       [ line_width_label#misc; line_width_spinner#misc;
 	turnstile_size_label#misc; turnstile_size_spinner#misc;
 	line_sep_label#misc; line_sep_spinner#misc;
+	proof_tree_sep_label#misc; proof_tree_sep_spinner#misc;
 	subtree_sep_label#misc; subtree_sep_spinner#misc;
 	command_length_label#misc; command_length_spinner#misc;
 	level_dist_label#misc; level_dist_spinner#misc;
+	layer_sep_label#misc; layer_sep_spinner#misc;
 	tree_font_label#misc; tree_font_button#misc;
 	sequent_font_label#misc; sequent_font_button#misc;
 	current_color_label#misc; current_color_button#misc;
@@ -1497,6 +1600,7 @@ let make_config_window () =
 	default_size_height_spinner#misc; 
 	internal_seq_lines_label#misc; internal_seq_lines_spinner#misc;
 	external_node_lines_label#misc; external_node_lines_spinner#misc;
+	ext_table_lines_label#misc; ext_table_lines_spinner#misc;
 	config_file_label#misc;
 	config_file_alignment#misc; debug_alignment#misc;
 	tee_file_box_alignment#misc;
