@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: draw_tree.ml,v 1.38 2012-05-14 14:03:36 tews Exp $
+ * $Id: draw_tree.ml,v 1.46 2013/01/20 21:55:54 tews Exp $
  *)
 
 
@@ -40,8 +40,8 @@
     tree is created in the right way.
 
     The common code of both proof-goal and proof-command nodes is in
-    the class {!proof_tree_element}. The class for proof goals,
-    {!turnstile} and the class {!proof_command} are derived from it.
+    the class {!class: Draw_tree.proof_tree_element}. The class for proof goals,
+    {!turnstile} and the class {!class: proof_command} are derived from it.
     To work around the impossible down-casts, {!proof_tree_element}
     contains some virtual method hooks for stuff that is really
     specific for just one of its subclasses.
@@ -59,7 +59,7 @@
     works top down. Therefore the nodes are organized in a
     doubly-linked tree, where children nodes contain a link to their
     parent. The doubly-linked tree functionality is in
-    {!doubly_linked_tree}. 
+    {!class: doubly_linked_tree}. 
 
 *)
 
@@ -96,7 +96,10 @@ open Gtk_ext
 *)
 
 
-(** Status of an existential variable *)
+(** Status of an existential variable. The tree of existentials is
+    only scanned for redisplay. Therefore, a fully instantiated
+    existential might have state [Partially_instantiated] until the next scan.
+*)
 type existential_status =
   | Uninstantiated			(** open, not instantiated *)
   | Partially_instantiated		(** instantiated, but the
@@ -105,7 +108,11 @@ type existential_status =
   | Fully_instantiated			(** fully instantiated *)
 
 
-(** Representation of existential variables. *)
+(** Representation of existential variables. The [status] field is
+    lazily updated in {!Proof_tree.update_existential_status}.
+    Therefore, a fully instantiated existential might have status
+    {!existential_status.Partially_instantiated} for some time.
+*)
 type existential_variable = {
   existential_name : string;		(** The name *)
   mutable status : existential_status;	(** instantiation status *)
@@ -342,6 +349,42 @@ let clear_children parent =
  *)
 
 
+(*****************************************************************************)
+(*****************************************************************************)
+(** {3 Tree layer interface} *)
+(*****************************************************************************)
+(*****************************************************************************)
+
+(** Abstract interface for {!class: Tree_layers.tree_layer} and
+    {!class: Tree_layers.tree_layer_stack}. Root nodes of proof trees and
+    layers contain a pointer to the layer or layer stack containing
+    them. This pointer is used to invalidate the size information in
+    these structures and to query location information. This class
+    type breaks the mutual dependency between root nodes and layers
+    and layers and the layer stack. The type parameter stands for the
+    structure containing the upward pointer, because it passes [self]
+    as first argument to {!child_offsets}.
+*)
+class type ['a] abstract_tree_container =
+object
+
+  (** Invalidate the size information in this container and all bigger
+      structures containing it.
+  *)
+  method clear_size_cache : unit
+
+  (** Compute the left and top offset of this container relative to
+      the upper-left corner of the complete display. 
+  *)
+  method left_top_offset : int * int
+
+  (** Compute the x and y offset of one child relative to the upper
+      left corner of this container.
+  *)
+  method child_offsets : 'a -> int * int
+end
+
+
 (*****************************************************************************)
 (*****************************************************************************)
 (** {3 External window interface} *)
@@ -393,11 +436,14 @@ end
     the code for (relativ) layout, (absolute) coordinates, locating
     mouse button clicks, marking branches and the general drawing
     functions.
+
+    Argument undo_state saves the undo state for the current proof.
+    It's value is arbitrary for cloned proof trees.
 *)
 class virtual proof_tree_element drawable
-    debug_name inst_existentials fresh_existentials = 
+    undo_state debug_name inst_existentials fresh_existentials = 
 object (self)
-  inherit [proof_tree_element] doubly_linked_tree as super
+  inherit [proof_tree_element] doubly_linked_tree
 
   (***************************************************************************)
   (***************************************************************************)
@@ -421,7 +467,12 @@ object (self)
   *)
   method inst_existentials : existential_variable list = inst_existentials
 
-  (** The {!Gtk_ext.better_drawable} into which this element draws itself. *)
+  (** Return the state for this sequent. *)
+  method undo_state = (undo_state : int)
+
+  (** The {!class: Gtk_ext.better_drawable} into which this element
+      draws itself.
+  *)
   val drawable = drawable
 
   (** The width of this node alone in pixels. Set in the initializer
@@ -472,6 +523,12 @@ object (self)
   (** The set of all existentials for this node. *)
   val mutable existential_variables = fresh_existentials
 
+  (** Upward pointer to the layer containing this proof tree. Must be
+      set for root nodes.
+  *)
+  val mutable tree_layer = 
+    (None : proof_tree_element abstract_tree_container option)
+
   (***************************************************************************)
   (***************************************************************************)
   (** {2 Accessors / Setters} *)
@@ -533,6 +590,10 @@ object (self)
       {!debug_name}. *)
   method virtual id : string
 
+  (** Register the proof tree layer containing this root node. *)
+  method register_tree_layer tl =
+    assert(tree_layer = None);
+    tree_layer <- Some tl
 
   (***************************************************************************)
   (***************************************************************************)
@@ -694,7 +755,11 @@ object (self)
      * then
      *)
     match parent with 
-      | None -> ()
+      | None -> 
+	(match tree_layer with
+	  | None -> assert false
+	  | Some sco -> sco#clear_size_cache
+	)
       | Some p -> p#update_sizes_in_branch
 
 
@@ -709,13 +774,23 @@ object (self)
   method child_offset child =
     self#iter_children 0 0 0 (fun left _y _a oc -> (left, child <> oc))
 
-  (** Computes the pair of the left offset and the offset of the
-      y-coordinate of this node relative to the upper-left corner of
-      the root node of the proof tree. 
+  (** Computes the pair [(left_off, y_off)]. [left_off] is the offset
+      of the left hand side of the bounding box of this node's
+      subtree. [y_off] is the offset of the y-coordinate of this node.
+      The offsets are relative to the left and top of the layer stack,
+      respectively.
   *)
   method left_y_offsets =
     match parent with
-      | None -> (0, height / 2)
+      | None -> 
+	(match tree_layer with
+	  | None -> assert false
+	  | Some tl -> 
+	    let (tl_left, tl_top) = tl#left_top_offset in
+	    let (me_left, me_top) = 
+	      tl#child_offsets (self :> proof_tree_element) in
+	    (tl_left + me_left, tl_top + me_top + height / 2)
+	)
       | Some p ->
 	let (parent_left, parent_y) = p#left_y_offsets in
 	let y_off = parent_y + !current_config.level_distance in
@@ -725,18 +800,24 @@ object (self)
 	(left_off, y_off)
 
   (** Computes the bounding box (that is a 4-tuple [(x_low, x_high,
-      y_low, y_high)]) relative to the upper-left corner of the root
-      node of the proof tree.
+      y_low, y_high)]) relative to the upper-left corner of the
+      complete display. 
   *)
   method bounding_box_offsets =
     let (left, y) = self#left_y_offsets in
+    let x = self#get_x_coordinate left in
+    (* 
+     * Printf.fprintf (debugc())
+     *   "BBO %s\n%!"
+     *   self#debug_name;
+     *)
     (* 
      * Printf.fprintf (debugc())
      *   "BBO left %d width %d height %d | x %d-%d y %d-%d\n%!"
      *   left width height 
      *   left (left + width) (y - height / 2) (y + height / 2);
      *)
-    (left, left + width, y - height / 2, y + height / 2)
+    (x - width / 2, x + width / 2, y - height / 2, y + height / 2)
 
   (** [bounding_box left top] computes the bounding box (that is a
       4-tuple [(x_low, x_high, y_low, y_high)]) of this node in
@@ -863,7 +944,7 @@ object (self)
       @param bx x-coordinate of point
       @param by y-coordinate of point
   *)
-  method mouse_button_tree left y bx by =
+  method find_node_for_point left y bx by =
     let top = y - height / 2 in
     if bx >= left && bx <= left + subtree_width &&
       by >= top && by <= top + self#subtree_height
@@ -876,7 +957,7 @@ object (self)
       else
 	self#iter_children left y None
 	  (fun left y _a child ->
-	    let cres = child#mouse_button_tree left y bx by in
+	    let cres = child#find_node_for_point left y bx by in
 	    (cres, cres = None))
     else
       None
@@ -894,8 +975,8 @@ object (self)
       @param bx x-coordinate of point
       @param by y-coordinate of point
   *)
-  method mouse_button_tree_root left top bx by =
-    self#mouse_button_tree left (top + height/2) bx by
+  method find_node_for_point_root left top bx by =
+    self#find_node_for_point left (top + height/2) bx by
 
 
   (***************************************************************************)
@@ -1069,13 +1150,13 @@ object (self)
       called when some existential got instantiated or when an undo
       uninstantiates some existential.
   *)
-  method update_existentials_display =
+  method update_existentials_info =
     (if external_windows <> [] && existential_variables <> [] 
      then
 	let new_text = self#displayed_text in
 	List.iter (fun ew -> ew#update_content new_text) external_windows
     );
-    List.iter (fun c -> c#update_existentials_display) children	
+    List.iter (fun c -> c#update_existentials_info) children	
 
   (** Hook to be called when the list of children has been changed.
       Adjusts the relative layout information of this element and all its
@@ -1092,6 +1173,7 @@ object (self)
       been changed. 
   *)
   method configuration_updated =
+    List.iter (fun c -> c#configuration_updated) children;
     List.iter (fun ex -> ex#configuration_updated) external_windows;
     self#set_node_size;
     self#update_subtree_size
@@ -1109,15 +1191,19 @@ end
 (** Specific element class for sequents, which draw themselves as
     turnstile symbols. This class specializes the abstract
     {!proof_tree_element} class for sequent nodes in the proof tree.
+
+    Argument undo_state saves the undo state for the current proof.
+    It's value is arbitrary for cloned proof trees.
 *)
-class turnstile (drawable : better_drawable) sequent_id sequent_text =
+class turnstile (drawable : better_drawable) 
+                undo_state sequent_id sequent_text =
 object (self)
-  inherit proof_tree_element drawable sequent_id [] [] as super
+  inherit proof_tree_element drawable undo_state sequent_id [] [] as super
 
   (** The pure sequent text. *)
   val mutable sequent_text = (sequent_text : string)
 
-  (** Pango layout for rendering text. Only created if there an ID for
+  (** Pango layout for rendering text. Only created if an ID for
       an external window must be put into the display.
   *)
   val mutable layout = None
@@ -1269,11 +1355,14 @@ let make_layout context =
 (** Specific element class for proof commands. This class specializes
     the generic and abstract {!proof_tree_element} for proof-command
     nodes.
+
+    Argument undo_state saves the undo state for the current proof.
+    It's value is arbitrary for cloned proof trees.
 *)
 class proof_command (drawable_arg : better_drawable) 
-  command debug_name inst_existentials fresh_existentials =
+  undo_state command debug_name inst_existentials fresh_existentials =
 object (self)
-  inherit proof_tree_element drawable_arg debug_name 
+  inherit proof_tree_element drawable_arg undo_state debug_name 
     inst_existentials fresh_existentials 
     as super
 
@@ -1343,12 +1432,15 @@ object (self)
     if Util.utf8_string_length command <= !current_config.proof_command_length
     then begin
       content_shortened <- false;
-      displayed_command <- command
+      displayed_command <- replace_char command '\n' ' '
     end else begin
       content_shortened <- true;
       displayed_command <-
-	(Util.utf8_string_sub command (!current_config.proof_command_length - 1))
-      ^ "\226\128\166" 			(* append horizontal ellipsis *)
+	(replace_char
+	   (Util.utf8_string_sub command 
+	      (!current_config.proof_command_length - 1))
+	   '\n' ' ')
+            ^ "\226\128\166" 			(* append horizontal ellipsis *)
     end
 
   (** Set {!proof_tree_element.width} and {!proof_tree_element.height}
@@ -1448,3 +1540,67 @@ object (self)
     ()
 
 end
+
+
+(*****************************************************************************)
+(*****************************************************************************)
+(** {2 Cloning} *)
+(*****************************************************************************)
+(*****************************************************************************)
+
+(** Helper for {!clone_tree_node} for cloning the existential
+    variables. Uses the hash to lookup variables, such that for each
+    variable only one {!existential_variable} record is created.
+*)
+let rec clone_existentials ex_hash ex =
+  try Hashtbl.find ex_hash ex.existential_name
+  with
+    | Not_found -> 
+      let deps = List.map (clone_existentials ex_hash) ex.dependencies in
+      let nex = { existential_name = ex.existential_name;
+		  status = ex.status;
+		  existential_mark = false;
+		  dependencies = deps;
+		}
+      in
+      Hashtbl.add ex_hash ex.existential_name nex;
+      nex
+
+(** Recursively clone all nodes in the given subtree, updating the
+    reference for the selected node, if the old selected node is contained
+    in the subtree.
+ *)
+let rec clone_tree_node new_pc new_seq ex_hash 
+    old_selected cloned_selected node =
+  let cloned_children = 
+    List.map 
+      (clone_tree_node new_pc new_seq ex_hash old_selected cloned_selected) 
+      node#children in
+  let clone = match node#node_kind with
+    | Proof_command -> 
+      (new_pc node#content 
+	 (List.map (clone_existentials ex_hash) node#inst_existentials)
+	 (List.map (clone_existentials ex_hash) node#fresh_existentials)
+	 : proof_command :> proof_tree_element)
+    | Turnstile -> 
+      (new_seq node#id node#content : turnstile :> proof_tree_element)
+  in
+  if Some node = old_selected
+  then cloned_selected := Some clone;
+  set_children clone cloned_children;
+  (match node#branch_state with
+    | Cheated
+    | Proven -> clone#set_branch_state node#branch_state
+    | Unproven
+    | CurrentNode
+    | Current -> ()
+  );
+  clone
+
+(** Clone a complete proof tree. *)
+let clone_proof_tree new_pc new_seq ex_hash old_selected cloned_selected root =
+  let cloned_root = 
+    clone_tree_node new_pc new_seq ex_hash old_selected cloned_selected root in
+  cloned_root#propagate_existentials;
+  cloned_root
+
