@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: input.ml,v 1.30 2012-05-14 14:03:37 tews Exp $
+ * $Id: input.ml,v 1.36 2013/01/17 20:32:01 tews Exp $
  *)
 
 
@@ -79,8 +79,9 @@
     the first message. 
     }
     {-  {v current-goals state %d current-sequent %s \
-    {cheated|not-cheated} proof-name-bytes %d command-bytes %d \
-    sequent-text-bytes %d additional-id-bytes %d existential-bytes %d\n\
+    {cheated|not-cheated} {new-layer|current-layer} proof-name-bytes %d \
+     command-bytes %d sequent-text-bytes %d additional-id-bytes %d \
+     existential-bytes %d\n\
     <data-proof-name>\n\
     <data-command>\n\
     <data-current-sequent>\n\
@@ -90,7 +91,7 @@
     The [current-goals] command tells [prooftree] about a new proof 
     state with a new set of open goals. This corresponds to either of
     the following cases:
-    {ul
+    {ol
     {- The initial proof state of a newly started proof}
     {- A proof command has been applied to the old current sequent, 
     yielding a new current sequent and possibly additional new 
@@ -98,14 +99,19 @@
     {- The old current goal has been solved (by some proof command) 
     and the new current sequent is one of the previously spawned 
     subgoals}
+    {- A new set of proof-tree root goal nodes is associated with the
+    current proof. This happens for instance, when Coq transformes
+    open existential variables into proof goals with the command 
+    [Grab Existential Variables].}
     }
     
-    [prooftree] decides with the help of its internal state which of 
-    the cases applies.
+    For case 1 or case 4 [new-layer] must be given. Otherwise,
+    [current-layer] must be specified and [prooftree] decides with the
+    help of its internal state whether case 2 or 3 applies.
     
-    The set of open goals does not need to represent the total 
-    set of all open subgoals, but it must contain all newly 
-    spawned subgoals.
+    For the second and the third case, the set of open goals does not
+    need to represent the total set of all open subgoals, but it must
+    contain all newly spawned subgoals.
     
     The state number in the [current-goals] command is for undo. It 
     is interpreted as the state that has been reached after processing 
@@ -156,14 +162,16 @@
     {- Full name of the proof}
     }
     }
-    {- {v proof-finished state %d {cheated|not-cheated} \
+    {- {v branch-finished state %d {cheated|not-cheated} \
     proof-name-bytes %d command-bytes %d existential-bytes %d\n\
     <data-proof-name>\n\
     <data-command>\n\
     <data-existentials>\n v}
     
-    [proof-finished] tells [prooftree] the last proof command that 
-    closed the last subgoal. The cheated flag tells [prooftree]
+    [branch-finished] tells [prooftree] the last proof command that 
+    closed the current branch. If there are still open subgoals, the 
+    proof will hopefully continue with one of them, which is not yet 
+    known. The cheated flag tells [prooftree]
     whether the new proof state was obtained by a cheating command 
     such as [admit] or [sorry]. The data sections are :
     {ol
@@ -191,7 +199,8 @@
     {- {v quit-proof proof-name-bytes %d\n\
     <data-proof-name>\n v}
     
-    Quit closes the window for the indicated proof.
+    Quit closes the window for the indicated proof. 
+    Cloned windows are not closed.
     The only data section is:
     {ol
     {- Full name of the proof whoose window should be delected}
@@ -203,7 +212,7 @@
 (** Version number of the communication protocol described and
     implemented by this module.
 *)
-let protocol_version = 2
+let protocol_version = 3
 
 
 
@@ -351,8 +360,13 @@ let local_input buf start len =
     @raise Sys_blocked_io when not enough input is available currently
 *)
 let rec get_string_cont s i len continuation_fn () =
+  (* Printf.fprintf (debugc()) "GS cont %d - %d enter\n%!" i len; *)
   current_parser := (get_string_cont s i len continuation_fn);
   let n = local_input s i (len - i) in
+  (* 
+   * Printf.fprintf (debugc()) "GS read %d bytes: %s\n%!" 
+   *   n (String.sub s i n);
+   *)
   let i = i + n in
   if i = len
   then begin
@@ -372,6 +386,7 @@ let rec get_string_cont s i len continuation_fn () =
     @raise Sys_blocked_io when not enough input is available currently
 *)
 let get_string len continuation_fn =
+  (* Printf.fprintf (debugc()) "GS %d enter\n%!" len; *)
   let s = String.create len in
   get_string_cont s 0 len continuation_fn ()
 
@@ -426,6 +441,7 @@ let parse_configure com_buf =
 (******************************************************************************
  ******************************************************************************
  * current-goals state %d current-sequent %s {cheated|not-cheated} \
+ * {new-layer|current-layer}
  * proof-name-bytes %d command-bytes %d sequent-text-bytes %d \
  * additional-id-bytes %d existential-bytes %d\n\
  * <data-proof-name>\n\
@@ -447,18 +463,21 @@ let parse_configure com_buf =
            line of the command
     @param cheated_string either "cheated" or "not-cheated" from the 
            first line of the command
+    @param layer_string either "new-layer" of "current-layer" from the
+           first line of the command
     @param proof_name name of the current proof
     @param proof_command text of the last proof command (or garbage if 
-           this is the first state of the proof)
+    this is the first state of the proof)
     @param current_sequent_text text of the current sequent
     @param additional_ids_string ID's of all currently open goals
     @param existentials_string prover specific information about 
-           existentials
+    existentials
 
 *)
 let parse_current_goals_finish state current_sequent_id cheated_string 
-    proof_name proof_command current_sequent_text 
+    layer_string proof_name proof_command current_sequent_text 
     additional_ids_string existentials_string =
+  (* Printf.fprintf (debugc()) "PCGF\n%!"; *)
   let cheated_flag = match cheated_string with
     | "not-cheated" -> false
     | "cheated" -> true
@@ -468,6 +487,15 @@ let parse_current_goals_finish state current_sequent_id cheated_string
 		  "Expected \"cheated\" or \"not-cheated\" as 6th word.",
 	       None))
   in
+  let layer_flag = match layer_string with
+    | "new-layer" -> true
+    | "current-layer" -> false
+    | _ ->
+      raise(Protocol_error
+	      ("Parse error in current-goals command. " ^
+		  "Expected \"new-layer\" or \"current-layer\" as 7th word.",
+	       None))
+  in
   let proof_name = chop_final_newlines proof_name in
   let proof_command = chop_final_newlines proof_command in
   let current_sequent_text = chop_final_newlines current_sequent_text in
@@ -476,7 +504,10 @@ let parse_current_goals_finish state current_sequent_id cheated_string
   let existentials_string = chop_final_newlines existentials_string in
   let (ex_uninst, ex_inst) = !parse_existential_info existentials_string in
   Proof_tree.process_current_goals state proof_name proof_command cheated_flag
-    current_sequent_id current_sequent_text additional_ids ex_uninst ex_inst
+    layer_flag current_sequent_id current_sequent_text additional_ids 
+    ex_uninst ex_inst;
+  current_parser := !message_start_parser
+
     
 
 (** Start parsing of the [current-goals] command. Extracts elements and 
@@ -487,11 +518,12 @@ let parse_current_goals_finish state current_sequent_id cheated_string
 let parse_current_goals com_buf =
   check_if_configured ();
   Scanf.bscanf com_buf 
-    (" state %d current-sequent %s %s proof-name-bytes %d "
+    (" state %d current-sequent %s %s %s proof-name-bytes %d "
      ^^ "command-bytes %d sequent-text-bytes %d "
      ^^ "additional-id-bytes %d existential-bytes %d")
-    (fun state current_sequent_id cheated_string proof_name_bytes command_bytes 
-      sequent_text_bytes additional_id_bytes existential_bytes ->
+    (fun state current_sequent_id cheated_string layer_string 
+      proof_name_bytes command_bytes sequent_text_bytes additional_id_bytes
+      existential_bytes ->
 	get_string proof_name_bytes
 	  (fun proof_name ->
 	    get_string command_bytes
@@ -503,7 +535,7 @@ let parse_current_goals com_buf =
 			get_string existential_bytes
 			  (fun existentials_string ->
 			    parse_current_goals_finish state current_sequent_id
-			      cheated_string
+			      cheated_string layer_string
 			      proof_name proof_command current_sequent_text
 			      additional_ids_string existentials_string))))))
 
@@ -531,7 +563,9 @@ let parse_current_goals com_buf =
 let parse_update_sequent_finish state sequent_id proof_name sequent_text =
   let proof_name = chop_final_newlines proof_name in
   let sequent_text = chop_final_newlines sequent_text in
-  Proof_tree.update_sequent state proof_name sequent_id sequent_text
+  Proof_tree.update_sequent state proof_name sequent_id sequent_text;
+  current_parser := !message_start_parser
+
 
 
 (** Parse and process a [update-sequent] command. Extracts the state and
@@ -570,7 +604,8 @@ let parse_update_sequent com_buf =
 *)
 let parse_switch_goal_finish state new_current_id proof_name =
   let proof_name = chop_final_newlines proof_name in
-  Proof_tree.switch_to state proof_name new_current_id
+  Proof_tree.switch_to state proof_name new_current_id;
+  current_parser := !message_start_parser
 
 
 (** Parse and process a [switch-goal] command. Extracts the state, the
@@ -590,18 +625,18 @@ let parse_switch_goal com_buf =
 
 
 (******************************************************************************
- * proof-finished state %d {cheated|not-cheated} \
+ * branch-finished state %d {cheated|not-cheated} \
  * proof-name-bytes %d command-bytes %d existential-bytes %d\n\
  * <data-proof-name>\n\
  * <data-command>\n\
  * <data-existentials>\n
 *)
 
-(** {3 Proof-finished command parser} *)
+(** {3 Branch-finished command parser} *)
 
 
-(** Finish parsing of the [proof-finished] command and process it
-    with {!Proof_tree.process_proof_finished}. The arguments are
+(** Finish parsing of the [branch-finished] command and process it
+    with {!Proof_tree.process_branch_finished}. The arguments are
 
     @param state state number
     @param cheated_string either "cheated" or "not-cheated"
@@ -609,14 +644,14 @@ let parse_switch_goal com_buf =
     @param proof_command last proof command (as raw data section string)
     @param existentials_string prover specific data about existentials
 *)
-let parse_proof_finished_finish 
+let parse_branch_finished_finish 
     state cheated_string proof_name proof_command existentials_string =
   let cheated_flag = match cheated_string with
     | "not-cheated" -> false
     | "cheated" -> true
     | _ -> 
       raise(Protocol_error
-	      ("Parse error in proof-finished command. " ^
+	      ("Parse error in branch-finished command. " ^
 		  "Expected \"cheated\" or \"not-cheated\" as 4th word.",
 	       None))
   in
@@ -624,16 +659,17 @@ let parse_proof_finished_finish
   let proof_command = chop_final_newlines proof_command in
   let existentials_string = chop_final_newlines existentials_string in
   let (ex_uninst, ex_inst) = !parse_existential_info existentials_string in
-  Proof_tree.process_proof_finished 
-    state proof_name proof_command cheated_flag ex_uninst ex_inst
+  Proof_tree.process_branch_finished 
+    state proof_name proof_command cheated_flag ex_uninst ex_inst;
+  current_parser := !message_start_parser
 
 
 (** Parse and process a [proof-finished] command. Extracts the
     necessary information from the first command line in the [Scanf]
     parsing buffer argument, reads the data section and finally calls
-    {!Input.parse_proof_finished_finish}.
+    {!Input.parse_branch_finished_finish}.
 *)
-let parse_proof_finished com_buf =
+let parse_branch_finished com_buf =
   check_if_configured ();
   Scanf.bscanf com_buf
     " state %d %s proof-name-bytes %d command-bytes %d existential-bytes %d"
@@ -645,7 +681,7 @@ let parse_proof_finished com_buf =
 	    (fun proof_command ->
 	      get_string existential_bytes
 		(fun existentials_string ->
-		  parse_proof_finished_finish 
+		  parse_branch_finished_finish 
 		    state cheated_string proof_name 
 		    proof_command existentials_string))))
 
@@ -670,12 +706,13 @@ let parse_proof_complete com_buf =
       get_string proof_name_bytes 
 	(fun proof_name ->
 	  let proof_name = chop_final_newlines proof_name in
-	  Proof_tree.process_proof_complete state proof_name))
+	  Proof_tree.process_proof_complete state proof_name;
+	  current_parser := !message_start_parser))
 
 
 (******************************************************************************
-									       * undo-to state %d\n
-*)
+ * undo-to state %d\n
+ *)
 
 (** {3 Undo-to command parser} *)
 
@@ -703,7 +740,8 @@ let do_undo com_buf =
 *)
 let parse_quit_proof_finish proof_name =
   let proof_name = chop_final_newlines proof_name in
-  Proof_tree.quit_proof proof_name
+  Proof_tree.quit_proof proof_name;
+  current_parser := !message_start_parser
 
 
 (** Parse and process a [quit-proof] command. Extracts the
@@ -743,7 +781,7 @@ let parse_command command =
       | "current-goals" -> parse_current_goals com_buf
       | "update-sequent" -> parse_update_sequent com_buf
       | "switch-goal" -> parse_switch_goal com_buf
-      | "proof-finished" -> parse_proof_finished com_buf
+      | "branch-finished" -> parse_branch_finished com_buf
       | "proof-complete" -> parse_proof_complete com_buf
       | "undo-to" -> do_undo com_buf
       | "quit-proof" -> parse_quit_proof com_buf
@@ -752,7 +790,6 @@ let parse_command command =
 			       None))
     );
   current_parser := !message_start_parser;
-  (* Printf.fprintf (debugc()) "PC finished\n%!"; *)
   ()
 
 
@@ -765,6 +802,7 @@ let parse_command command =
 let read_second_line first_line =
   Scanf.sscanf first_line "second line %3d\n"
     (fun second_line_len ->
+      (* Printf.fprintf (debugc()) "second line cont %d\n%!" second_line_len; *)
       get_string second_line_len parse_command)
 
 
@@ -783,6 +821,7 @@ let message_start () =
   (* every message starts with a line "second line %03d"
    * where the number gives the bytes in the next line
    *)
+  (* Printf.fprintf (debugc()) "message start\n%!"; *)
   try
     get_string 16 read_second_line
   with
