@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: proof_tree.ml,v 1.38 2012-05-14 14:03:37 tews Exp $
+ * $Id: proof_tree.ml,v 1.50 2013/01/20 21:55:54 tews Exp $
  *)
 
 
@@ -34,6 +34,7 @@ open Util
 open Gtk_ext
 open Configuration
 open Draw_tree
+open Tree_layers
 open Proof_window
 open Emacs_commands
 
@@ -58,7 +59,7 @@ type proof_tree = {
   proof_name : string; 
   (** The name of the proof *)
 
-  pa_start_state : int;
+  mutable pa_start_state : int;
   (** Internal proof assistant state number where this proof starts. 
       Used to detect bulk undos.
   *)
@@ -74,18 +75,21 @@ type proof_tree = {
 
   sequent_hash : (string, turnstile) Hashtbl.t;
   (** Hash table mapping all currently known sequents of this proof
-      tree to {!Draw_tree.turnstile} objects. Used to detect new
+      tree to {!class: Draw_tree.turnstile} objects. Used to detect new
       sequents and to update sequents.
   *)
 
-  mutable current_sequent_id : string;
-  (** The ID of the current sequent. Needed to distinguish the
-      peculiar case, where a non-failing proof command (such as
-      [auto]) does not change the proof state. 
+  mutable current_sequent_id : string option;
+  (** The ID of the current sequent, if there is one. Needed to
+      distinguish the peculiar case, where a non-failing proof command
+      (such as [auto]) does not change the proof state. 
   *)
 
-  mutable current_sequent : proof_tree_element;
-  (** The object of the current sequent. Used for coloring branches. *)
+  mutable current_sequent : proof_tree_element option;
+  (** The object of the current sequent, if there is one. Used for
+      coloring branches. There is no current sequent, if a branch has
+      been finished without switching to a new goal.
+  *)
 
   mutable open_goals_count : int;
   (** Counter for the total number of open goals, including the
@@ -116,9 +120,11 @@ type proof_tree = {
 (** State record for displayed proof trees. The code maintains the
     following invariants.
     {ul 
-    {- Each live state is in precisely one the lists
-    {!all_proof_trees_for_undo}, {!undo_surviver_trees} or
+    {- Each displayed proof tree is in precisely one the lists
+    {!original_proof_trees} or
     {!Proof_window.cloned_proof_windows}.}
+    {- {!proof_tree.current_sequent_id} = [None] iff
+    {!proof_tree.current_sequent} = [None]}
     }
 *)
 
@@ -139,20 +145,10 @@ let add_undo_action pt pa_state undo_fun =
 	pt.undo_actions <- (pa_state, [undo_fun]) :: pt.undo_actions
 
 
-(** Contains all proof trees that should be affected by undo
-    operations. Cloned proof trees and proof trees that survived a
-    bulk undo are not in this list.
+(** Contains all non-cloned proof trees managed in this module.
+    Cloned proof trees are in {!Proof_window.cloned_proof_windows}.
 *)
-let all_proof_trees_for_undo = ref []
-
-(** Contains proof trees that survived a bulk undo. Like cloned proof
-    trees these trees do not take part in undo actions. Unlike cloned
-    proof trees these proof trees should be reused, when the user
-    eventually starts on the proof again.
-
-    (There is another list cloned_proof_windows in proof_window.ml.)
-*)
-let undo_surviver_trees = ref []
+let original_proof_trees = ref []
 
 
 (** Take the necessary actions when the configuration record changed.
@@ -161,9 +157,7 @@ let undo_surviver_trees = ref []
 *)
 let configuration_updated () =
   List.iter (fun pt -> pt.window#configuration_updated) 
-    !all_proof_trees_for_undo;
-  List.iter (fun pt -> pt.window#configuration_updated) 
-    !undo_surviver_trees;
+    !original_proof_trees;
   List.iter (fun ptw -> ptw#configuration_updated)
     !cloned_proof_windows
 
@@ -241,7 +235,7 @@ let update_existential_status ex_hash =
     of those existentials that got instantiated is updated. 
 
     This function returns the list of newly instantiated existentials
-    and the list of new uninstantiated existentials.
+    and the list of new existentials.
 *)
 let update_existentials ex_hash uninst_ex inst_ex_deps =
   let test_and_create_ex_list exl accu =
@@ -253,8 +247,8 @@ let update_existentials ex_hash uninst_ex inst_ex_deps =
       )
       accu exl
   in
-  let new_uninst = test_and_create_ex_list uninst_ex [] in
-  let new_uninst =
+  let new_ex = test_and_create_ex_list uninst_ex [] in
+  let new_ex =
     List.fold_left
       (fun res (ex_name, deps) ->
 	(* Complex stategies might create and instantiate several
@@ -264,7 +258,7 @@ let update_existentials ex_hash uninst_ex inst_ex_deps =
 	 *)
 	test_and_create_ex_list (ex_name :: deps) res
       )
-      new_uninst inst_ex_deps
+      new_ex inst_ex_deps
   in
   let ex_got_instatiated =
     List.fold_left
@@ -280,7 +274,7 @@ let update_existentials ex_hash uninst_ex inst_ex_deps =
       [] inst_ex_deps
   in
   (* XXX use a coq specific comparison function for sorting *)
-  (ex_got_instatiated, List.sort compare new_uninst)
+  (ex_got_instatiated, List.sort compare new_ex)
 	 
 
 
@@ -289,10 +283,18 @@ let update_existentials ex_hash uninst_ex inst_ex_deps =
     sequent area if there is no selected node.
 *)
 let set_current_node_wrapper pt sequent =
-  pt.window#set_current_node sequent;
+  (match sequent with
+     | Some s -> pt.window#set_current_node s;
+     | None -> pt.window#clear_current_node;
+  );
   if pt.window#get_selected_node = None then
     pt.sequent_area_needs_refresh <- true
 
+(** Local convenience function for marking the current sequent. *)
+let mark_current_sequent_maybe = function
+  | Some cs -> cs#mark_current
+  | None -> ()
+
 
 (** Holds the state for the currently active proof window, if any.
     Mainly used for {!finish_drawing} to delay redrawing.
@@ -304,9 +306,11 @@ let current_proof_tree = ref None
     Additionally clear {!current_proof_tree}.
 *)
 let stop_proof_tree pt pa_state = 
-  (* Keep undo actions, never know if the prover supports jumping in
-   * to the middle of a proof.
+  (* Don't keep undo actions. Because of insertions, undo state
+   * numbers might get out of sync with retired proof trees. Undo into
+   * the middle of a proof is therefore impossible.
    *)
+  pt.undo_actions <- [];
   pt.pa_end_state <- pa_state;
   pt.window#disconnect_proof;
   pt.window#clear_current_node;
@@ -325,7 +329,10 @@ let stop_proof_tree pt pa_state =
 let stop_proof_tree_last_selected pt pa_state =
   (match pt.window#get_selected_node with
     | None -> 
-      add_undo_action pt pa_state (fun () -> pt.window#set_selected_node None);
+      (* stop_proof_tree will clear all undo actions
+       * add_undo_action pt pa_state
+       * 	(fun () -> pt.window#set_selected_node None);
+       *)
       pt.window#select_root_node
     | Some _ -> ());
   stop_proof_tree pt pa_state
@@ -338,7 +345,6 @@ type proof_tree_undo_result =
   | PT_undo_delete    (** Argument proof tree should be deleted  *)
   | PT_undo_current   (** Argument proof tree should be kept as current *)
   | PT_undo_keep      (** Argument proof tree should be kept non-current *)
-  | PT_undo_surviver  (** Argument proof tree should be kept as surviver *)
 
 
 (** Process all undo actions with a state greater than [undo_state].
@@ -367,15 +373,26 @@ let rec fire_undo_actions undo_state = function
     a bulk undo with a state less than the starting state of [pt] was
     detected.)}
     }
+    Because of insertions, undo-state numbers might get out of sync
+    with retired proof trees. Therefore, undo into the middle of an
+    retired proof tree is not supported.
 *)
 let undo_tree pt pa_state =
+  let pt_is_current = match !current_proof_tree with
+    | Some cpt -> pt == cpt
+    | None -> false
+  in
   if pa_state < pt.pa_start_state
   then begin
     if pt.window#survive_undo_before_start 
     then begin
       pt.window#message "Retract before start";
-      stop_proof_tree pt (-1);
-      PT_undo_surviver
+      if pt_is_current then
+	stop_proof_tree pt (-1)
+      else 
+	pt.pa_end_state <- -1;
+      pt.window#clear_position_hints;
+      PT_undo_keep
     end
     else begin
       pt.window#delete_proof_window;
@@ -383,28 +400,46 @@ let undo_tree pt pa_state =
     end
   end 
   else if pt.pa_end_state >= 0 && pa_state >= pt.pa_end_state 
-  then PT_undo_keep
-  else begin
-    pt.pa_end_state <- -1;
-    pt.undo_actions <- fire_undo_actions pa_state pt.undo_actions;
-    pt.current_sequent#mark_current;
-    set_current_node_wrapper pt pt.current_sequent;
-    pt.window#message (Printf.sprintf "Retract to %d" pa_state);
-    pt.need_redraw <- true;
-    PT_undo_current
+  then begin
+    assert(pt_is_current = false);
+    PT_undo_keep
   end
+  else (* pt.pa_start_state <= pa_state < pt.pa_end_state *)
+    if pt_is_current 
+    then
+      begin
+	pt.undo_actions <- fire_undo_actions pa_state pt.undo_actions;
+	mark_current_sequent_maybe pt.current_sequent;
+	set_current_node_wrapper pt pt.current_sequent;
+	pt.window#message (Printf.sprintf "Retract to %d" pa_state);
+	pt.need_redraw <- true;
+	pt.window#clear_position_hints;
+	PT_undo_current
+      end
+    else 
+      begin
+	if pt.window#survive_undo_before_start 
+	then begin
+	  pt.window#message "Retract before start";
+	  pt.pa_end_state <- -1;
+	  pt.window#clear_position_hints;
+	  PT_undo_keep
+	end
+	else begin
+	  pt.window#delete_proof_window;
+	  PT_undo_delete
+	end
+      end 
 
 
-(** Perform undo to state [pa_state] in all proof trees currently
-    affected by undo ({!all_proof_trees_for_undo}). As result some of
-    the proof windows might get deleted, some proof trees might get
-    changed, and some might be kept as surviver (in
-    {!undo_surviver_trees}). {!current_proof_tree} might be cleared or
-    changed.
+(** Perform undo to state [pa_state] in all non-cloned proof trees
+    ({!original_proof_trees}). As result some of the proof windows
+    might get deleted, some proof trees might get changed, and some
+    might be kept as surviver. {!current_proof_tree} might be cleared.
 *)
 let undo pa_state =
   let new_current = ref None in
-  all_proof_trees_for_undo :=
+  original_proof_trees :=
     List.fold_left
     (fun pts pt -> match undo_tree pt pa_state with
       | PT_undo_delete -> pts
@@ -412,117 +447,148 @@ let undo pa_state =
 	new_current := Some pt;
 	pt :: pts
       | PT_undo_keep -> pt :: pts
-      | PT_undo_surviver -> 
-	undo_surviver_trees := pt :: !undo_surviver_trees;
-	pts
     )
-    [] !all_proof_trees_for_undo;
+    [] !original_proof_trees;
   current_proof_tree := !new_current
 
 
-(** Try to find a surviver proof window for [proof_name]. If found, it
-    is dequeued from the surviver list ({!undo_surviver_trees}) and
-    returned. (The caller will then reuse it.)
+(** Try to find an old proof window for [proof_name]. 
 *)
 let get_surviver proof_name =
-  let rec doit res = function
-    | [] -> None
-    | pt :: pts ->
-      if pt.proof_name = proof_name
-      then begin
-	undo_surviver_trees := List.rev_append res pts;
-	Some pt
-      end 
-      else doit (pt::res) pts
-  in
-  doit [] !undo_surviver_trees
+  try
+    Some(
+      List.find (fun pt -> pt.proof_name = proof_name) !original_proof_trees
+    )
+  with
+    | Not_found -> None
 
 
-(** Create a new proof-tree state (especially a new proof-tree window)
-    for [proof_name] with starting state [state] and initial sequent
-    [current_sequent].
+(** Create a new proof-tree state (containing an empty proof tree
+    window) for [proof_name] with starting state [state].
 *)
-let create_new_proof_tree proof_name state 
-    current_sequent_id current_sequent_text =
-  let pt_win = make_proof_window proof_name !geometry_string in
-  let sw = pt_win#new_turnstile current_sequent_id current_sequent_text in
-  let hash = Hashtbl.create 503 in
-  Hashtbl.add hash current_sequent_id sw;
-  let sw = (sw :> proof_tree_element) in
-  let pt = {
-    window = pt_win;
+let create_new_proof_tree proof_name state =
+  {
+    window = make_proof_window proof_name !geometry_string;
     proof_name = proof_name;
     pa_start_state = state;
     pa_end_state = -1;
     cheated = false;
-    sequent_hash = hash;
-    current_sequent_id = current_sequent_id;
-    current_sequent = sw;
-    open_goals_count = 1;
+    sequent_hash = Hashtbl.create 503;
+    current_sequent_id = None;
+    current_sequent = None;
+    open_goals_count = 0;
     existential_hash = Hashtbl.create 251;
     need_redraw = true;
     sequent_area_needs_refresh = true;
     undo_actions = [];
-  } in
-  pt_win#set_root sw;
-  pt
+  }
 
 
 (** Initialize a surviver proof-tree state (and window) for reuse with
     the initial sequent [current_sequent] and start state [state].
 *)
-let reuse_surviver pt state current_sequent_id current_sequent_text =
-  let pt_win = pt.window in
-  let proof_name = pt.proof_name in
-  let sequent_hash = pt.sequent_hash in
-  let ex_hash = pt.existential_hash in
-  let sw = pt_win#new_turnstile current_sequent_id current_sequent_text in
-  pt_win#clear_for_reuse;
-  Hashtbl.clear sequent_hash;
-  Hashtbl.clear ex_hash;
-  Hashtbl.add pt.sequent_hash current_sequent_id sw;
-  let sw = (sw :> proof_tree_element) in
-  let pt = {
-    window = pt_win;
-    proof_name = proof_name;
-    pa_start_state = state;
-    pa_end_state = -1;
-    cheated = false;
-    sequent_hash = sequent_hash;
-    current_sequent_id = current_sequent_id;
-    current_sequent = sw;
-    open_goals_count = 1;
-    existential_hash = ex_hash;
-    need_redraw = true;
-    sequent_area_needs_refresh = true;
-    undo_actions = [];
-  } in
-  pt_win#set_root sw;
-  pt_win#message "";
-  pt
+let reinit_surviver pt state =
+  pt.window#clear_for_reuse;
+  Hashtbl.clear pt.sequent_hash;
+  Hashtbl.clear pt.existential_hash;
+  pt.pa_start_state <- state;
+  pt.pa_end_state <- -1;
+  pt.cheated <- false;
+  pt.current_sequent_id <- None;
+  pt.current_sequent <- None;
+  pt.open_goals_count <- 0;
+  pt.need_redraw <- true;
+  pt.sequent_area_needs_refresh <- true;
+  pt.undo_actions <- [];
+  pt.window#message ""
 
 
-(** Start a new proof [proof_name] with [current_sequent] as initial
-    sequent in state [state]. If a surviver proof tree is found it is
-    reused. Otherwise a new proof-tree state and window is created.
+(** Start a new proof [proof_name] which is initially empty, that is
+    contains no proof tree layers. If a surviver proof tree is found
+    it is reused. Otherwise a new proof-tree state and window is
+    created.
 *)
-let start_new_proof state proof_name current_sequent_id current_sequent_text =
-  assert(List.for_all 
-	   (fun pt -> pt.proof_name <> proof_name) !all_proof_trees_for_undo);
+let start_new_proof state proof_name =
   let pt =
     match get_surviver proof_name with
       | None -> 
-	create_new_proof_tree proof_name state 
-	  current_sequent_id current_sequent_text
+	let pt = create_new_proof_tree proof_name state in
+	original_proof_trees := pt :: !original_proof_trees;
+	pt
       | Some pt -> 
-	reuse_surviver pt state current_sequent_id current_sequent_text
+	reinit_surviver pt state;
+	pt
   in
-  pt.current_sequent#mark_current;
-  set_current_node_wrapper pt pt.current_sequent;
-  pt.window#message "Initial sequent";
-  pt.need_redraw <- true;
   current_proof_tree := Some pt;
-  all_proof_trees_for_undo := pt :: !all_proof_trees_for_undo
+  pt
+
+
+(** Create a new layer in the proof tree display with the current
+    sequent and all additional sequents as root goals. There must be
+    no open subgoal. The information about existential variables is
+    processed, but there must be no new and no newly instantiated
+    existential variables. If this is the first layer in the display,
+    a warning is displayed, if there are more than 1 root nodes.
+*)
+let create_new_layer pt state current_sequent_id current_sequent_text
+    additional_ids uninstantiated_existentials instantiated_ex_deps =
+  assert(List.for_all (fun id -> Hashtbl.mem pt.sequent_hash id = false)
+	   (current_sequent_id :: additional_ids));
+  assert(pt.open_goals_count = 0);
+  let (ex_got_instantiated, new_existentials) =
+    update_existentials pt.existential_hash 
+      uninstantiated_existentials instantiated_ex_deps in
+  assert (ex_got_instantiated = [] && new_existentials = []);
+  let first_sw = 
+    pt.window#new_turnstile state current_sequent_id current_sequent_text in
+  Hashtbl.add pt.sequent_hash current_sequent_id first_sw;
+  let first_sw = (first_sw :> proof_tree_element) in
+  let other_sw =
+    List.fold_right
+      (fun id res ->
+	let sw = pt.window#new_turnstile state id "waiting for sequent text" in
+	Hashtbl.add pt.sequent_hash id sw;
+	(sw :> proof_tree_element) :: res)
+      additional_ids []
+  in
+  let all_roots = first_sw :: other_sw in
+  let layer = new tree_layer all_roots in
+  let layer_undo_pos = pt.window#layer_stack#add_layer layer in
+  pt.current_sequent_id <- Some current_sequent_id;
+  pt.current_sequent <- Some first_sw;
+  pt.open_goals_count <- List.length all_roots;
+  first_sw#mark_current;
+  set_current_node_wrapper pt (Some first_sw);
+  pt.window#clear_position_hints;
+  let layer_count = pt.window#layer_stack#count_layers in
+  let message =
+    if layer_count = 1
+    then "Initial sequent"
+    else 
+      Printf.sprintf "Layer %d with %d goals" layer_count pt.open_goals_count
+  in
+  pt.window#message message;
+  let unhash_sequent_ids = current_sequent_id :: additional_ids in
+  let undo () =
+    List.iter (fun s -> s#delete_non_sticky_external_windows) all_roots;
+    List.iter (fun id -> Hashtbl.remove pt.sequent_hash id) unhash_sequent_ids;
+    List.iter 
+      (fun sw -> if sw#is_selected then pt.window#set_selected_node None)
+      all_roots;
+    pt.window#layer_stack#del_layer layer_undo_pos;
+    pt.current_sequent_id <- None;
+    pt.current_sequent <- None;
+    pt.open_goals_count <- 0;
+  in
+  add_undo_action pt state undo;
+  pt.need_redraw <- true;
+  pt.sequent_area_needs_refresh <- true;
+  if layer_count = 1 && pt.open_goals_count > 1 then
+    run_message_dialog
+      "More than one initial goal. \n\
+       You need to start proofs with \n\
+       the \"Proof\" command!"
+      `WARNING
 
 
 (** Add a new proof command with the new current sequent
@@ -546,13 +612,18 @@ let add_new_goal pt state proof_command cheated_flag current_sequent_id
     update_existentials pt.existential_hash 
       uninstantiated_existentials instantiated_ex_deps
   in
+  let parent = match pt.current_sequent with
+    | Some s -> s
+    | None -> assert false
+  in
   let pc = 
     pt.window#new_proof_command 
-      proof_command ex_got_instantiated new_existentials
+      state proof_command ex_got_instantiated new_existentials
   in
   let pc = (pc :> proof_tree_element) in
-  set_children pt.current_sequent [pc];
-  let sw = pt.window#new_turnstile current_sequent_id current_sequent_text in
+  set_children parent [pc];
+  let sw =
+    pt.window#new_turnstile state current_sequent_id current_sequent_text in
   Hashtbl.add pt.sequent_hash current_sequent_id sw;
   let sw = (sw :> proof_tree_element) in
   let new_goal_ids_rev = 
@@ -562,29 +633,37 @@ let add_new_goal pt state proof_command cheated_flag current_sequent_id
   let new_goals =
     List.fold_left
       (fun res id ->
-	let sw = pt.window#new_turnstile id "waiting for sequent text" in
+	let sw = pt.window#new_turnstile state id "waiting for sequent text" in
 	Hashtbl.add pt.sequent_hash id sw;
 	let sw = (sw :> proof_tree_element) in
 	sw :: res)
       [] new_goal_ids_rev
   in
+  let position_hints = match new_goals with
+    | [] -> [[pc]]
+    | [snd] -> [[snd; pc]; [snd]; [pc]]
+    | snd :: rest -> 
+      let last = list_last rest in
+      [[last; pc]; [snd; pc]; [snd]; [pc]]
+  in
   let all_subgoals = sw :: new_goals in
   set_children pc all_subgoals;
   let unhash_sequent_ids = current_sequent_id :: new_goal_ids_rev in
   let old_current_sequent_id = pt.current_sequent_id in
-  let old_current_sequent = pt.current_sequent in
+  let old_current_sequent = parent in
   let old_open_goals_count = pt.open_goals_count in
-  pt.current_sequent_id <- current_sequent_id;
-  pt.current_sequent <- sw;
+  pt.current_sequent_id <- Some current_sequent_id;
+  pt.current_sequent <- Some sw;
   pt.open_goals_count <- pt.open_goals_count + List.length new_goals;
   sw#mark_current;
-  set_current_node_wrapper pt sw;
+  set_current_node_wrapper pt (Some sw);
+  pt.window#set_position_hints position_hints;
   (* The uninstantiated existentials are displayed together with the
    * sequent. Therefore, if some existential got instantiated we have
    * to update all those sequent displays.
    *)
   if ex_got_instantiated <> [] then begin
-    pt.window#update_existentials_display;
+    pt.window#update_sequent_existentials_info;
     pt.sequent_area_needs_refresh <- true;
   end;
   let message = match List.length new_goals with
@@ -608,11 +687,11 @@ let add_new_goal pt state proof_command cheated_flag current_sequent_id
       all_subgoals;
     if pc#is_selected then pt.window#set_selected_node None;
     pt.current_sequent_id <- old_current_sequent_id;
-    pt.current_sequent <- old_current_sequent;
+    pt.current_sequent <- Some old_current_sequent;
     pt.open_goals_count <- old_open_goals_count;
     if ex_got_instantiated <> [] then begin
       undo_instantiate_existentials ex_got_instantiated;
-      pt.window#update_existentials_display;
+      pt.window#update_sequent_existentials_info;
       pt.sequent_area_needs_refresh <- true;
     end;
     List.iter (fun ex -> Hashtbl.remove pt.existential_hash ex.existential_name)
@@ -635,28 +714,35 @@ let finish_branch pt state proof_command cheated_flag
     update_existentials pt.existential_hash 
       uninstantiated_existentials instantiated_ex_deps
   in
+  let parent = match pt.current_sequent with
+    | Some s -> s
+    | None -> assert false
+  in
   let pc = 
     pt.window#new_proof_command 
-      proof_command ex_got_instantiated new_existentials
+      state proof_command ex_got_instantiated new_existentials
   in
   let pc = (pc :> proof_tree_element) in
-  pt.current_sequent#unmark_current;
-  set_children pt.current_sequent [pc];
+  parent#unmark_current;
+  set_children parent [pc];
   if cheated_flag 
   then pc#mark_cheated
   else pc#mark_proved;
   let old_cheated = pt.cheated in
-  let old_current_sequent = pt.current_sequent in
+  let old_current_sequent = parent in
+  let old_current_sequent_id = pt.current_sequent_id in
   let old_open_goals_count = pt.open_goals_count in
   let undo () =
     pc#delete_non_sticky_external_windows;
     clear_children old_current_sequent;
     old_current_sequent#unmark_proved_or_cheated;
+    pt.current_sequent <- Some old_current_sequent;
+    pt.current_sequent_id <- old_current_sequent_id;
     pt.open_goals_count <- old_open_goals_count;
     pt.cheated <- old_cheated;
     if ex_got_instantiated <> [] then begin
       undo_instantiate_existentials ex_got_instantiated;
-      pt.window#update_existentials_display;
+      pt.window#update_sequent_existentials_info;
       pt.sequent_area_needs_refresh <- true;
     end;
     List.iter (fun ex -> Hashtbl.remove pt.existential_hash ex.existential_name)
@@ -666,8 +752,12 @@ let finish_branch pt state proof_command cheated_flag
   add_undo_action pt state undo;
   if cheated_flag then pt.cheated <- true;
   pt.open_goals_count <- pt.open_goals_count - 1;
+  pt.current_sequent <- None;
+  pt.current_sequent_id <- None;
+  pt.window#clear_position_hints;
+  set_current_node_wrapper pt None;
   if ex_got_instantiated <> [] then begin
-    pt.window#update_existentials_display;
+    pt.window#update_sequent_existentials_info;
     pt.sequent_area_needs_refresh <- true;
   end;
   pt.window#ext_dialog_add new_existentials;
@@ -678,21 +768,21 @@ let finish_branch pt state proof_command cheated_flag
     the current one. 
 *)
 let internal_switch_to pt state new_current_sequent_id =
+  assert(pt.current_sequent = None && pt.current_sequent_id = None);
   let new_current_sequent = 
     Hashtbl.find pt.sequent_hash new_current_sequent_id 
   in
   let new_current_sequent = (new_current_sequent :> proof_tree_element) in
-  let old_current_sequent_id = pt.current_sequent_id in
-  let old_current_sequent = pt.current_sequent in
   let undo () =
     new_current_sequent#unmark_current;
-    pt.current_sequent_id <- old_current_sequent_id;
-    pt.current_sequent <- old_current_sequent;
+    pt.current_sequent_id <- None;
+    pt.current_sequent <- None;
   in
   new_current_sequent#mark_current;
-  set_current_node_wrapper pt new_current_sequent;
-  pt.current_sequent_id <- new_current_sequent_id;
-  pt.current_sequent <- new_current_sequent;
+  set_current_node_wrapper pt (Some new_current_sequent);
+  pt.current_sequent_id <- Some new_current_sequent_id;
+  pt.current_sequent <- Some new_current_sequent;
+  pt.window#clear_position_hints;
   add_undo_action pt state undo;
   pt.need_redraw <- true
 
@@ -722,30 +812,37 @@ let finish_branch_and_switch_to pt state proof_command cheated_flag
 
 (* See mli for doc *)
 let process_current_goals state proof_name proof_command cheated_flag
-    current_sequent_id current_sequent_text additional_ids 
+    layer_flag current_sequent_id current_sequent_text additional_ids 
     uninstatiated_existentials instantiated_ex_deps =
   (match !current_proof_tree with
     | Some pt -> 
       if pt.proof_name <> proof_name 
       then stop_proof_tree_last_selected pt state
     | None -> ());
-  match !current_proof_tree with
-    | None -> 
-      assert(additional_ids = []);
-      assert(cheated_flag = false);
-      assert(uninstatiated_existentials = []);
-      start_new_proof state proof_name current_sequent_id current_sequent_text
+  let layer_flag = layer_flag || !current_proof_tree = None in
+  let pt = match !current_proof_tree with
+    | None -> start_new_proof state proof_name
     | Some pt ->
-      if pt.current_sequent_id <> current_sequent_id &&
-	Hashtbl.mem pt.sequent_hash current_sequent_id
-      then
-	finish_branch_and_switch_to pt state proof_command cheated_flag
-	  current_sequent_id additional_ids 
-	  uninstatiated_existentials instantiated_ex_deps
-      else
-	add_new_goal pt state proof_command cheated_flag current_sequent_id 
-	  current_sequent_text additional_ids 
-	  uninstatiated_existentials instantiated_ex_deps
+      assert ((iff (pt.current_sequent = None) (pt.current_sequent_id = None))
+	      && (iff layer_flag (pt.current_sequent = None)));
+      pt
+  in
+  if layer_flag
+  then begin
+    assert (cheated_flag = false);
+    create_new_layer pt state current_sequent_id current_sequent_text
+      additional_ids uninstatiated_existentials instantiated_ex_deps
+  end else
+    if pt.current_sequent_id <> (Some current_sequent_id) &&
+      Hashtbl.mem pt.sequent_hash current_sequent_id
+    then
+      finish_branch_and_switch_to pt state proof_command cheated_flag
+	current_sequent_id additional_ids 
+	uninstatiated_existentials instantiated_ex_deps
+    else
+      add_new_goal pt state proof_command cheated_flag current_sequent_id 
+	current_sequent_text additional_ids 
+	uninstatiated_existentials instantiated_ex_deps
 
 
 (** Update the sequent text for some sequent. This function is used
@@ -783,6 +880,29 @@ let update_sequent state proof_name sequent_id sequent_text =
 	  raise (Proof_tree_error "Update unknown sequent")
 
 
+(** Leave current branch as is to prepare for switching to a different
+    goal.
+*)
+let leave_branch pt state =
+  assert (pt.current_sequent <> None && pt.current_sequent_id <> None);
+  let last_node = match pt.current_sequent with
+    | Some s -> s
+    | None -> assert false
+  in
+  last_node#unmark_current;
+  let old_current_sequent = last_node in
+  let old_current_sequent_id = pt.current_sequent_id in
+  let undo () =
+    pt.current_sequent <- Some old_current_sequent;
+    pt.current_sequent_id <- old_current_sequent_id;
+  in
+  add_undo_action pt state undo;
+  pt.current_sequent <- None;
+  pt.current_sequent_id <- None;
+  set_current_node_wrapper pt None;
+  pt.need_redraw <- true
+
+
 (* See mli for doc *)
 let switch_to state proof_name new_current_sequent_id =
   match !current_proof_tree with
@@ -791,33 +911,71 @@ let switch_to state proof_name new_current_sequent_id =
     | Some pt ->
       if pt.proof_name <> proof_name
       then raise (Proof_tree_error "Switch to sequent on other proof");
-      pt.current_sequent#unmark_current;
-      internal_switch_to pt state new_current_sequent_id;
-      let message = 
-	Printf.sprintf "Branch changed (%d goal%s remaining)" 
-	  pt.open_goals_count
-	  (if pt.open_goals_count > 1 then "s" else "")
-      in
-      pt.window#message message
+      match pt.current_sequent_id with
+      | Some old_id ->
+	assert (pt.current_sequent <> None);
+	(* Coq generates a lot switch-to commands if you use bullets
+	 * and braces. Some of them don't actually change the current goal.
+	 *)
+	if old_id <> new_current_sequent_id then begin
+	  leave_branch pt state;
+	  internal_switch_to pt state new_current_sequent_id;
+	  let message = 
+	    Printf.sprintf "Branch changed (%d goal%s remaining)" 
+	      pt.open_goals_count
+	      (if pt.open_goals_count > 1 then "s" else "")
+	  in
+	  pt.window#message message
+	end
+      | None ->
+	assert (pt.current_sequent = None);
+	internal_switch_to pt state new_current_sequent_id;
+	let message =
+	  Printf.sprintf "%d open goal%s"
+	    pt.open_goals_count (if pt.open_goals_count > 1 then "s" else "")
+	in
+	pt.window#message message
 
 
 (* See mli for doc *)
-let process_proof_finished state proof_name proof_command cheated_flag
+let process_branch_finished state proof_name proof_command cheated_flag
     uninstatiated_existentials instantiated_ex_deps =
   match !current_proof_tree with
     | None -> 
-      raise (Proof_tree_error "proof-finished without current proof tree")
+      raise (Proof_tree_error "branch-finished without current proof tree")
     | Some pt -> 
       if pt.proof_name <> proof_name
-      then raise (Proof_tree_error "Finish other non-current proof");
+      then raise (Proof_tree_error "Branch finish in other proof");
+      assert (pt.current_sequent <> None && pt.current_sequent_id <> None);
       finish_branch pt state proof_command cheated_flag
 	uninstatiated_existentials instantiated_ex_deps;
       let message = 
-	if pt.cheated 
-	then pango_markup_bold_color "False proof finished" 
-	  !cheated_gdk_color
-	else pango_markup_bold_color "Proof finished" 
-	  !proved_complete_gdk_color
+	if pt.open_goals_count = 0
+	then begin
+	  let all_ex_inst = 
+	    Hashtbl.fold (fun _ ex res -> res && ex.status <> Uninstantiated)
+	      pt.existential_hash true
+	  in
+	  let message_text =
+	    (if pt.cheated then "False proof finished" else "Proof finished")
+	    ^ (if all_ex_inst then "" else " (incomplete)")
+	  in
+	  let color = 
+	    if pt.cheated then !cheated_gdk_color
+	    else 
+	      if all_ex_inst then !proved_complete_gdk_color
+	      else !proved_incomplete_gdk_color
+	  in
+	  pango_markup_bold_color message_text color
+	end else 
+	  Printf.sprintf "%s (%d goal%s remaining)" 
+	    (if cheated_flag
+	     then pango_markup_bold_color "Branch aborted" 
+		!cheated_gdk_color
+	     else pango_markup_bold_color "Branch finished" 
+		!proved_complete_gdk_color)
+	    pt.open_goals_count
+	    (if pt.open_goals_count > 1 then "s" else "")
       in
       pt.window#message message
 
@@ -854,10 +1012,8 @@ let clear_proof_tree_lists proof_name =
     end
     else pt :: pts
   in
-  all_proof_trees_for_undo := 
-    List.fold_left proof_tree_list_fold_fun [] !all_proof_trees_for_undo;
-  undo_surviver_trees :=
-    List.fold_left proof_tree_list_fold_fun [] !undo_surviver_trees
+  original_proof_trees := 
+    List.fold_left proof_tree_list_fold_fun [] !original_proof_trees
 
 let quit_proof proof_name =
   (match !current_proof_tree with 
