@@ -19,7 +19,7 @@
  * You should have received a copy of the GNU General Public License
  * along with "prooftree". If not, see <http://www.gnu.org/licenses/>.
  * 
- * $Id: proof_window.ml,v 1.47 2012-05-14 14:03:37 tews Exp $
+ * $Id: proof_window.ml,v 1.59 2013/01/21 14:26:36 tews Exp $
  *)
 
 
@@ -29,7 +29,9 @@
 open Util
 open Gtk_ext
 open Configuration
+open Emacs_commands
 open Draw_tree
+open Tree_layers
 open Node_window
 open Help_window
 open About_window
@@ -44,8 +46,7 @@ let delete_proof_tree_callback = ref (fun (_ : string) -> ())
 
 
 (** Contains proof window clones. See also
-    [Proof_tree.all_proof_trees_for_undo] and
-    [Proof_tree.undo_surviver_trees].
+    [Proof_tree.original_proof_trees].
 *)
 let cloned_proof_windows = ref []
 
@@ -74,14 +75,15 @@ let cloned_proof_windows = ref []
                               of the vertical scroll bar of the sequent area
     - message_label           {xref lablgtk class GMisc.label}
                               for messages in the bottom line
-    - menu                    {xref lablgtk class GMenu.menu} of main menu
+    - context_menu            {xref lablgtk class GMenu.menu} 
+                              of the context menu
     - proof_name              the name of the proof this display is showing
 *)
 class proof_window (top_window : GWindow.window)
   drawing_h_adjustment drawing_v_adjustment (drawing_area : GMisc.drawing_area)
   (drawable : better_drawable)
   labeled_sequent_frame sequent_window sequent_v_adjustment
-  (message_label : GMisc.label) menu proof_name
+  (message_label : GMisc.label) context_menu proof_name
   =
 object (self)
 
@@ -91,15 +93,12 @@ object (self)
    *
    ***************************************************************************)
 
-  (** x-offset of the bounding box of the complete proof tree. This is
-      only non-zero when the width of the complete proof tree is smaller
-      than the width of the drawing area.
-  *)
+  (** x-offset of the bounding box of the complete proof tree layer
+      stack. This is only non-zero when the width of the complete
+      layer stack is smaller than the width of the drawing area. *)
   val mutable top_left = 0
 
-  (** y-offset of the bounding box of the root node of the proof tree.
-      Constantly 0 because the root node stays at the top.
-  *)
+  (** y-offset of the bounding box of the top layer. Constantly 0. *)
   val top_top = 0
 
   (** [true] if the sequent area should always show the last line.
@@ -107,8 +106,8 @@ object (self)
       a proof command is shown. *)
   val mutable sequent_window_scroll_to_bottom = false
 
-  (** The root node of the proof-tree, if it exists. *)
-  val mutable root = None
+  (** The stack of layers of proof trees *)
+  val layer_stack = new tree_layer_stack
 
   (** The current node of the proof tree, if there is one *)
   val mutable current_node = None
@@ -129,6 +128,13 @@ object (self)
   *)
   val mutable position_to_current_node = true
 
+  (** List of position hints, containing lists of proof tree nodes. If
+      possible, the current node will positioned such that some hint
+      nodes are also visible. If the list is longer than one, hints
+      are tried one after each other. 
+  *)
+  val mutable position_hints = []
+
   (** Holds the selected node, if there is one. *)
   val mutable selected_node = None
 
@@ -153,12 +159,18 @@ object (self)
   *)
   val mutable destroy_in_progress = false
 
-  (** Set the root node of the proof tree. *)
-  method set_root r = 
-    root <- Some (r : proof_tree_element)
+  (** True when this is a clone. *)
+  val mutable is_clone = false
 
-  (** Clear the root node of the proof tree. *)
-  method clear_root = root <- None
+  (** A proof is disconnected if it is not the current proof that is
+      currently updated by Proof General. Used for the context menu.
+  *)
+  val mutable disconnected = false
+
+  (** The stack of layers, containing all proof trees belonging to
+      this proof.
+  *)
+  method layer_stack = layer_stack
 
   (** Return the selected node or [None] if there is none. *)
   method get_selected_node = selected_node
@@ -167,6 +179,15 @@ object (self)
   method delete_node_window win =
     node_windows <- List.filter (fun owin -> owin <> win) node_windows
 
+  (** Setter for {!is_clone}. *)
+  method set_clone_flag = is_clone <- true
+
+  (** Setter for {!position_hints}. *)
+  method set_position_hints hints =
+    position_hints <- (hints : proof_tree_element list list)
+
+  (** Clear {!position_hints}. *)
+  method clear_position_hints = position_hints <- []
 
   (***************************************************************************
    *
@@ -270,32 +291,22 @@ object (self)
       window shows more than just the root sequent, it will stay
       alive.
   *)
-  method survive_undo_before_start =
-    match root with
-      | None -> false
-      | Some root -> root#children <> []
+  method survive_undo_before_start = 
+    layer_stack#survive_undo_before_start_hint
 
   (** Disconnect the proof tree. Clears all current node and current
       branch indications.
   *)
-  method disconnect_proof =
-    match root with
-      | None -> ()
-      | Some root -> root#disconnect_proof
+  method disconnect_proof = 
+    disconnected <- true;
+    layer_stack#disconnect
 
   (** Reflect changes in the current configuration. *)
   method configuration_updated =
-    let rec update_tree_sizes node =
-      List.iter update_tree_sizes node#children;
-      node#configuration_updated
-    in
     sequent_window#misc#modify_font !sequent_font_desc;
     drawable#set_line_attributes 
       ~width:(!current_config.turnstile_line_width) ();
-    (match root with
-      | None -> ()
-      | Some root -> update_tree_sizes root
-    );
+    layer_stack#configuration_updated;
     self#expand_drawing_area;
     ignore(self#position_tree);
     GtkBase.Widget.queue_draw top_window#as_widget;
@@ -306,7 +317,8 @@ object (self)
       Deletes and destroys all non-sticky external node windows.
   *)
   method clear_for_reuse =
-    root <- None;
+    disconnected <- false;
+    layer_stack#clear_for_reuse;
     current_node <- None;
     current_node_offset_cache <- None;
     selected_node <- None;
@@ -315,32 +327,14 @@ object (self)
     assert(node_windows = []);
     self#clear_existential_dialog_for_reuse
 
-  (** Update the existential dialog, if there is one. *)
-  method update_existentials_display =
-    match root with
-      | None -> ()
-      | Some root -> root#update_existentials_display
+  (** Update the existentials info in external sequent displays. *)
+  method update_sequent_existentials_info =
+    layer_stack#update_sequent_existentials_info
 
   (** Find the first node in the proof tree satisfying the predicate
       using depth-first search.
   *)
-  method find_node p =
-    let res = ref None in
-    let rec iter node =
-      if p node 
-      then begin
-	res := Some node;
-	raise Exit
-      end
-      else List.iter iter node#children
-    in
-    (match root with
-      | None -> ()
-      | Some root ->
-	try iter root
-	with Exit -> ()
-    );
-    !res
+  method find_node p = layer_stack#find_node p
 
 
   (***************************************************************************
@@ -432,18 +426,6 @@ object (self)
    *
    ***************************************************************************)
 
-  (** Scroll the given adjustment [direction] number of pages into the
-      direction idicated by the sign of [direction]. This method is
-      used for scrolling with keys.
-  *)
-  method private scroll (adjustment : GData.adjustment) direction =
-    let a = adjustment in
-    let new_val = a#value +. float_of_int(direction) *. a#step_increment in
-    let new_val = if new_val < 0.0 then 0.0 else new_val in
-    let max = max 0.0 (a#upper -. a#page_size) in
-    let new_val = if new_val > max then max else new_val in
-    a#set_value new_val
-
   (** Delete and destroy this proof-tree window, including all its
       non-sticky external node windows and the existential variable
       dialog. Delete this window from {!cloned_proof_windows}. Other
@@ -474,7 +456,8 @@ object (self)
   *)
   method user_delete_proof_window () =
     if destroy_in_progress = false then begin
-      !delete_proof_tree_callback proof_name;
+      if is_clone = false then
+	!delete_proof_tree_callback proof_name;
       self#delete_proof_window
     end
 
@@ -491,18 +474,16 @@ object (self)
       | ks when (ks = GdkKeysyms._Q or ks = GdkKeysyms._q)  -> 
 	self#user_delete_proof_window (); true
       | ks when ks = GdkKeysyms._Left -> 
-	self#scroll drawing_h_adjustment (-1); true
+	scroll_adjustment drawing_h_adjustment (-1); true
       | ks when ks = GdkKeysyms._Right -> 
-	self#scroll drawing_h_adjustment 1; true
+	scroll_adjustment drawing_h_adjustment 1; true
       | ks when ks = GdkKeysyms._Up -> 
-	self#scroll drawing_v_adjustment (-1); true
+	scroll_adjustment drawing_v_adjustment (-1); true
       | ks when ks = GdkKeysyms._Down -> 
-	self#scroll drawing_v_adjustment 1; true
+	scroll_adjustment drawing_v_adjustment 1; true
 
-      (* 
-       * | ks when (ks = GdkKeysyms._E or ks = GdkKeysyms._e)  -> 
-       * 	self#show_existential_window (); true
-       *)
+      | ks when (ks = GdkKeysyms._E or ks = GdkKeysyms._e)  -> 
+      	self#show_existential_window (); true
 
       (* 
        * | ks when (ks = GdkKeysyms._C or ks = GdkKeysyms._c)  -> 
@@ -532,10 +513,8 @@ object (self)
 	let ext = 
 	  make_ext_dialog (self :> proof_window) proof_name 
 	in
-	(match root with
-	  | None -> ()
-	  | Some root_node -> ext#fill_table root_node
-	);
+	layer_stack#init_ext_dialog ext;
+	ext#finish_table_init;
 	existential_window <- Some ext
 
   (** Destroy the existential variables dialog, if one is present. *)
@@ -656,10 +635,8 @@ object (self)
 	    y_page_size >= (y_u_off - y_l_off)
 	  then begin
 	    (* current node fits into the viewport, be sophisticated *)
-	    let (req_width, req_height) = match root with
-	      | None -> (0,0)
-	      | Some root -> (root#subtree_width, root#subtree_height)
-	    in
+	    let req_width = layer_stack#width in
+	    let req_height = layer_stack#height in
 	    if (float_of_int req_width) > drawing_h_adjustment#upper ||
 	       (float_of_int req_height) > drawing_v_adjustment#upper
 	    then begin
@@ -680,7 +657,38 @@ object (self)
 		  (float_of_int (y_u_i - y_page_size))
 	      in
 	      drawing_v_adjustment#set_value y_val;
-	      drawing_h_adjustment#clamp_page ~lower:x_l_f ~upper:x_u_f;
+
+	      (* make now the _h_adjustment, first try if one of the 
+               * hint nodes can be made visible as well
+	       *)
+	      if 
+		List.exists
+		  (fun hint_nodes ->
+		    let (off_l, off_u) =
+		      List.fold_left
+			(fun (off_l, off_u) hint_node ->
+			  let (h_x_l_off, h_x_u_off, _, _) = 
+			    hint_node#bounding_box_offsets 
+			  in
+			  (min off_l h_x_l_off, max off_u h_x_u_off))
+			(x_l_off, x_u_off)
+			hint_nodes
+		    in
+		    if x_page_size >= (off_u - off_l)
+		    then begin
+		      (* the hints fit *)
+		      drawing_h_adjustment#clamp_page
+			~lower:(float_of_int (top_left + off_l))
+			~upper:(float_of_int (top_top + off_u));
+		      true
+		    end else 
+		      (* the hints don't fit *)
+		      false
+		  )
+		  position_hints
+	      then ()
+	      else
+		drawing_h_adjustment#clamp_page ~lower:x_l_f ~upper:x_u_f;
 	      (* 
                * Printf.fprintf (debugc()) "clever y_u_i %d up %d y_val %d %!"
 	       * 	y_u_i
@@ -728,36 +736,29 @@ object (self)
       current proof tree.
   *)
   method private expand_drawing_area =
-    match root with
-      | None -> ()
-      | Some root -> 
-	let new_width = root#subtree_width in
-	let new_height = root#subtree_height in
-	(* 
-         * Printf.fprintf (debugc()) "DRAWING AREA SIZE REQUEST %d x %d\n%!" 
-	 *   new_width new_height;
-         *)
-	(* 
-         * if new_width > current_width || new_height > current_height then
-	 *   drawing_area#misc#set_size_request
-	 *     ~width:(max current_width new_width)
-	 *     ~height:(max current_height new_height) ();
-         *)
-	drawing_area#misc#set_size_request
-	  ~width:new_width ~height:new_height ();
+    let new_width = layer_stack#width in
+    let new_height = layer_stack#height in
+    (* 
+     * Printf.fprintf (debugc()) "DRAWING AREA SIZE REQUEST %d x %d\n%!" 
+     *   new_width new_height;
+     *)
+    (* 
+     * if new_width > current_width || new_height > current_height then
+     *   drawing_area#misc#set_size_request
+     * 	~width:(max current_width new_width)
+     * 	~height:(max current_height new_height) ();
+     *)
+    drawing_area#misc#set_size_request ~width:new_width ~height:new_height ();
 
   (** Sets the position of the proof tree in the drawing area by 
       computing [top_left]. Returns true if the position changed. 
       In that case the complete drawing area must be redrawn.
   *)
   method private position_tree =
-    match root with
-      | None -> false
-      | Some root -> 
-	let old_top_left = top_left in
-	let (width, _) = drawable#size in
-	top_left <- max 0 ((width - root#subtree_width) / 2);
-	top_left <> old_top_left
+    let old_top_left = top_left in
+    let (width, _) = drawable#size in
+    top_left <- max 0 ((width - layer_stack#width) / 2);
+    top_left <> old_top_left
 
   (** Handle expose events. Try to move to the current node, erase and
       redraw the complete drawing area.
@@ -773,12 +774,8 @@ object (self)
      *)
     self#try_adjustment;
     self#erase;
-    (* let left = 0 in *)
-    match root with
-      | None -> ()
-      | Some root ->
-	(* Printf.fprintf (debugc()) "REDRAW\n%!"; *)
-	ignore(root#draw_tree_root top_left top_top)
+    (* Printf.fprintf (debugc()) "REDRAW\n%!"; *)
+    layer_stack#draw top_left top_top
 
   (** Schedule an expose event for the drawing area, thereby causing
       it to get redrawn.
@@ -989,7 +986,7 @@ object (self)
       if there is no root node
   *)
   method select_root_node =
-    self#set_selected_node root
+    self#set_selected_node (layer_stack#get_root_node)
 
   (** Save the selected node state on a button press and clear
       {!restored_selected_node}. 
@@ -1020,12 +1017,8 @@ object (self)
   method private locate_button_node : 
     'a . int -> int -> (#proof_tree_element -> 'a) -> (unit -> 'a) -> 'a =
     fun x y node_click_fun outside_click_fun ->
-      let node_opt = match root with 
-	| None -> None
-	| Some root ->
-	  root#mouse_button_tree_root top_left top_top x y
-      in
-      match node_opt with
+      match layer_stack#find_node_for_point_in_layer_stack 
+	                              top_left top_top x y with
 	| None -> outside_click_fun ()
 	| Some node -> node_click_fun node
 
@@ -1081,18 +1074,23 @@ object (self)
      * last_button_press_time <- GdkEvent.Button.time ev;
      *)
     (* 
-     * Printf.fprintf (debugc()) "%s Button %s%d at %d x %d\n%!" 
-     *   (match GdkEvent.get_type ev with
-     * 	| `BUTTON_PRESS -> "single"
-     * 	| `TWO_BUTTON_PRESS -> "double"
-     * 	| `THREE_BUTTON_PRESS -> "triple"
-     * 	| `BUTTON_RELEASE -> "release")
-     *   (if shifted then "shift " else "")
-     *   button x y;
+     * (let h = drawing_h_adjustment in
+     *  let v = drawing_v_adjustment in
+     *  Printf.fprintf (debugc()) 
+     *    "%s Button %s%d at %d x %d | x: %1.f - %.1f y: %.1f - %.1f\n%!" 
+     *    (match GdkEvent.get_type ev with
+     * 	 | `BUTTON_PRESS -> "single"
+     * 	 | `TWO_BUTTON_PRESS -> "double"
+     * 	 | `THREE_BUTTON_PRESS -> "triple"
+     * 	 | `BUTTON_RELEASE -> "release")
+     *    (if shifted then "shift " else "")
+     *    button x y
+     *    h#value (h#value +. h#page_size)
+     *    v#value (v#value +. v#page_size));
      *)
     (match button with
       | 1 -> self#button_1_press x y shifted double
-      | 3 -> menu#popup ~button ~time:(GdkEvent.Button.time ev)
+      | 3 -> self#context_menu x y button (GdkEvent.Button.time ev)
       | _ -> ());
     true
 
@@ -1145,6 +1143,105 @@ object (self)
     else false
 
 
+  (***************************************************************************
+   *
+   * menu actions
+   *
+   ***************************************************************************)
+
+  (** Field to store the undo state number. This number is determined
+      when the context menu is posted and is needed in the callback for
+      the undo menu entry. *)
+  val mutable context_menu_undo_state = None
+
+  (** Field to store the proof command under the mouse when the
+      context menu is posted. This node is needed for sending proof
+      commands and proof scripts. To avoid a memory leak, this field
+      is cleared with an idle action, see {!context_menu_deactivated}.
+  *)
+  val mutable context_menu_node = None
+
+  (** Callback for the undo menu entry. *)
+  method undo_to_point () = 
+    match context_menu_undo_state with
+      | None -> assert false
+      | Some state -> emacs_callback_undo state
+
+  (** Callback for inserting one proof command. *)
+  method insert_proof_command () = 
+    match context_menu_node with
+      | None -> assert false
+      | Some node -> emacs_send_proof_script node#content
+
+  (** Callback for inserting all proof commands of a subtree. *)
+  method insert_subproof () =
+    let buf = Buffer.create 4095 in
+    let rec collect node indent =
+      Buffer.add_string buf (String.make indent ' ');
+      Buffer.add_string buf node#content;
+      Buffer.add_char buf '\n';
+      let sequents = node#children in
+      ignore(
+	List.fold_left
+	  (fun indent sequent ->
+	    match sequent#children with
+	      | [pc] -> collect pc indent; indent - 2
+	      | [] -> indent - 2        (* happens for the current sequent *)
+	      | _ -> assert false)
+	  (indent + (2 * List.length sequents) - 2)
+	  sequents)
+    in
+    match context_menu_node with
+      | None -> assert false
+      | Some node -> 
+	collect node 0;
+	emacs_send_proof_script (Buffer.contents buf)
+
+  (** Post the context menu. Depending on where the mouse button is
+      pressed, certain menu entries as disabled. The undo state is
+      stored in {!context_menu_undo_state} for use in the callback for
+      the undo entry.
+  *)
+  method context_menu x y button time =
+    self#locate_button_node x y
+      (fun node -> 
+	context_menu_node <- Some node;
+	let undo_state = 
+	  if disconnected then None
+	  else
+	    match node#node_kind with
+	      | Proof_command -> None
+	      | Turnstile -> match node#children with
+		  | first :: _ -> Some first#undo_state
+		  | _ -> None
+	in
+	context_menu_undo_state <- undo_state;
+	(List.nth context_menu#children 0)#misc#set_sensitive
+	  (undo_state <> None);
+	(List.nth context_menu#children 1)#misc#set_sensitive
+	  (node#node_kind = Proof_command);
+	(List.nth context_menu#children 2)#misc#set_sensitive
+	  (node#node_kind = Proof_command);
+	context_menu#popup ~button ~time;
+      )
+      (fun () ->
+	List.iter
+	  (fun n ->
+	    (List.nth context_menu#children n)#misc#set_sensitive false)
+	  [0; 1; 2];
+	context_menu#popup ~button ~time
+      )
+
+  (** Callback for the deactivated signal of the context menu.
+      Unfortunately, this signal is processed before the callback for
+      the selected menu item is processed. Therefore, the
+      {!context_menu_node} cannot be cleared directly but only in an
+      idle action.
+  *)
+  method context_menu_deactivated () =
+    ignore(GMain.Idle.add (fun () -> context_menu_node <- None; false))
+
+
   (***************************************************************************
    *
    * tooltips
@@ -1187,56 +1284,16 @@ object (self)
       window.
   *)
   method clone (owin : proof_window) =
-    let become_selected = match current_node with
+    owin#set_clone_flag;
+    let old_selected = match current_node with
       | Some _ -> current_node
       | None -> selected_node
     in
     let cloned_selected = ref None in
-    let ex_hash = Hashtbl.create 251 in
-    let rec copy_existential ex =
-      try Hashtbl.find ex_hash ex.existential_name
-      with
-	| Not_found -> 
-	  let deps = List.map copy_existential ex.dependencies in
-	  let nex = { existential_name = ex.existential_name;
-		      status = ex.status;
-		      existential_mark = false;
-		      dependencies = deps;
-		    }
-	  in
-	  Hashtbl.add ex_hash ex.existential_name nex;
-	  nex
-    in
-    let rec clone_tree node =
-      let cloned_children = List.map clone_tree node#children in
-      let clone = match node#node_kind with
-	| Proof_command -> 
-	  (owin#new_proof_command node#content 
-	     (List.map copy_existential node#inst_existentials)
-	     (List.map copy_existential node#fresh_existentials)
-	   :> proof_tree_element)
-	| Turnstile -> 
-	  (owin#new_turnstile node#id node#content :> proof_tree_element)
-      in
-      if Some node = become_selected
-      then cloned_selected := Some clone;
-      set_children clone cloned_children;
-      (match node#branch_state with
-	| Cheated
-	| Proven -> clone#set_branch_state node#branch_state
-	| Unproven
-	| CurrentNode
-	| Current -> ()
-      );
-      clone
-    in    
-    (match root with
-      | None -> ()
-      | Some root_node ->
-	let cloned_tree = clone_tree root_node in
-	cloned_tree#propagate_existentials;
-	owin#set_root cloned_tree
-    );
+    let cloned_layers = 
+      layer_stack#clone_layers (owin#new_proof_command 0) (owin#new_turnstile 0)
+        old_selected cloned_selected in
+    owin#layer_stack#set_layers cloned_layers;
     owin#set_selected_node !cloned_selected;
     owin#refresh_and_position;
     cloned_proof_windows := owin :: !cloned_proof_windows
@@ -1249,12 +1306,13 @@ object (self)
    ***************************************************************************)
 
   (** Create a new turnstile node for sequents. *)
-  method new_turnstile sequent_id sequent_text =
-    new turnstile drawable sequent_id sequent_text
+  method new_turnstile undo_state sequent_id sequent_text =
+    new turnstile drawable undo_state sequent_id sequent_text
 
   (** Create a new proof command node. *)
-  method new_proof_command command inst_existentials new_existentials =
-    new proof_command drawable command command 
+  method new_proof_command undo_state command 
+                           inst_existentials new_existentials =
+    new proof_command drawable undo_state command command 
       (inst_existentials : existential_variable list)
       (new_existentials : existential_variable list)
 end
@@ -1339,18 +1397,42 @@ let rec make_proof_window name geometry_string =
     GButton.button ~label:"Menu" ~packing:(button_h_box#pack) ()
   in
 
-  let menu = GMenu.menu () in
+  let main_menu = GMenu.menu () in
+
+  let context_menu = GMenu.menu () in
 
   let proof_window = 
     new proof_window top_window 
       drawing_h_adjustment drawing_v_adjustment drawing_area
       drawable labeled_sequent_frame sequent_window sequent_v_adjustment
-      message_label menu name
+      message_label context_menu name
   in
   let clone_fun () =
     let owin = make_proof_window name geometry_string in
     proof_window#clone owin
   in
+
+  let main_menu_entries = 
+    [`I("Clone", clone_fun);
+     `I("Show current", proof_window#reposition_current_node);
+     `I("Existentials", proof_window#show_existential_window);
+     `I("Configuration", show_config_window);
+     `I("Help", show_help_window);
+     `I("About", show_about_window);
+     `I("Exit", (fun _ -> exit 0));
+    ] in
+  let context_menu_entries =
+    [`I("Undo to point", proof_window#undo_to_point);
+     `I("Insert command", proof_window#insert_proof_command);
+     `I("Insert subproof", proof_window#insert_subproof);
+     `S
+    ] @ main_menu_entries
+  in
+  GToolbox.build_menu main_menu ~entries:main_menu_entries;
+  GToolbox.build_menu context_menu ~entries:context_menu_entries;
+  ignore(context_menu#connect#deactivate 
+           ~callback:(proof_window#context_menu_deactivated));
+
   top_window#set_title (name ^ " proof tree");
   drawable#set_line_attributes 
     ~width:(!current_config.turnstile_line_width) ();
@@ -1405,19 +1487,9 @@ let rec make_proof_window name geometry_string =
 	   ~callback:proof_window#user_delete_proof_window);
   ignore(menu_button#connect#clicked 
 	   ~callback:(fun () -> 
-	     menu#popup ~button:0 
+	     main_menu#popup ~button:0 
 	       ~time:(GtkMain.Main.get_current_event_time ())));
 
-  GToolbox.build_menu menu
-    ~entries:[`I("Clone", clone_fun);
-	      `I("Show current", proof_window#reposition_current_node);
-	      `I("Existentials", proof_window#show_existential_window);
-	      `I("Configuration", show_config_window);
-	      `I("Help", show_help_window);
-	      `I("About", show_about_window);
-	      `I("Exit", (fun _ -> exit 0));
-	     ];
-
   top_window#show ();
   if geometry_string <> "" then
     ignore(top_window#parse_geometry geometry_string);
